/*
================================================================================
  RPL/2 (R) version 4.1.21
  Copyright (C) 1989-2015 Dr. BERTRAND Jol

  This file is part of RPL/2.

  RPL/2 is free software; you can redistribute it and/or modify it
  under the terms of the CeCILL V2 License as published by the french
  CEA, CNRS and INRIA.
 
  RPL/2 is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the CeCILL V2 License
  for more details.
 
  You should have received a copy of the CeCILL License
  along with RPL/2. If not, write to info@cecill.info.
================================================================================
*/


#include "rpl-conv.h"


/*
================================================================================
  Procdure d'estimation de la longueur du tampon
================================================================================
  Entre :
--------------------------------------------------------------------------------
  Sortie :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

static inline void
estimation_taille_pile(struct_processus *s_etat_processus)
{
	/*
	 * Cette fonction permet d'estimer un volant de structures de maillons
	 * de liste chane pour le programme courant et vite un certain nombre
	 * d'allocations de mmoire lors des manipulations de la pile. Cette taille
	 * est estime au travers d'une chane de Markov.
	 */

	if ((*s_etat_processus).hauteur_pile_operationnelle >
			(*s_etat_processus).estimation_taille_pile_tampon)
	{
		(*s_etat_processus).estimation_taille_pile_tampon =
				(*s_etat_processus).estimation_taille_pile_tampon;
	}
	else
	{
		(*s_etat_processus).estimation_taille_pile_tampon =
				(((double) (*s_etat_processus).estimation_taille_pile_tampon) *
				((double) 0.95)) + (((double) (*s_etat_processus)
				.hauteur_pile_operationnelle) * ((double) 0.05));
	}

	return;
}


/*
================================================================================
  Procdure d'empilement d'un nouvel lment
================================================================================
  Entre :
--------------------------------------------------------------------------------
  Sortie :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

logical1
empilement(struct_processus *s_etat_processus,
		struct_liste_chainee **l_base_pile, struct_objet *s_objet)
{
	struct_liste_chainee		*l_ancienne_base_liste;
	struct_liste_chainee		*l_nouvelle_base_liste;

	logical1					erreur;

	l_ancienne_base_liste = *l_base_pile;

	if ((*s_etat_processus).pile_tampon == NULL)
	{
		// Tampon vide, on alloue un lment.

		if ((l_nouvelle_base_liste = malloc(sizeof(struct_liste_chainee)))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return(d_erreur);
		}
	}
	else
	{
		// Tampon utilisable, on retire un lment du tampon.

		l_nouvelle_base_liste = (*s_etat_processus).pile_tampon;
		(*s_etat_processus).pile_tampon = (*l_nouvelle_base_liste).suivant;
		(*s_etat_processus).taille_pile_tampon--;
	}

	*l_base_pile = l_nouvelle_base_liste;
	(**l_base_pile).donnee = s_objet;
	(**l_base_pile).suivant = l_ancienne_base_liste;

	erreur = d_absence_erreur;

/*
-- Ne considre que la pile oprationnelle -------------------------------------
*/

	if ((*s_etat_processus).l_base_pile == *l_base_pile)
	{
		(*s_etat_processus).hauteur_pile_operationnelle++;
		estimation_taille_pile(s_etat_processus);

		if ((*s_etat_processus).debug == d_vrai)
			if (((*s_etat_processus).type_debug &
					d_debug_pile_utilisateur) != 0)
		{
			if ((*s_etat_processus).langue == 'F')
			{
				printf("[%d] Empilement de type %d "
						"(profondeur %lld)\n", (int) getpid(),
						(*s_objet).type,
						(*s_etat_processus).hauteur_pile_operationnelle);
			}
			else
			{
				printf("[%d] Pushing a type %d object "
						"(depth %lld)\n", (int) getpid(), (*s_objet).type,
						(*s_etat_processus).hauteur_pile_operationnelle);
			}

			fflush(stdout);
		}
	}

	return erreur;
}


/*
================================================================================
  Procdure de dpilement d'un lment. L'emplacement est libr dans la pile.
================================================================================
  Entre :
--------------------------------------------------------------------------------
  Sortie :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

logical1
depilement(struct_processus *s_etat_processus,
		struct_liste_chainee **l_base_pile, struct_objet **s_objet)
{
	struct_liste_chainee		*l_ancienne_base_liste;
	struct_liste_chainee		*l_nouvelle_base_liste;

	logical1					erreur;

	if (*l_base_pile == NULL)
	{
		(*s_etat_processus).erreur_execution = d_ex_pile_vide;
		erreur = d_erreur;
	}
	else
	{

/*
-- Ne considre que la pile oprationnelle -------------------------------------
*/

		l_ancienne_base_liste = *l_base_pile;
		*s_objet = (*l_ancienne_base_liste).donnee;

		if ((*s_etat_processus).l_base_pile == *l_base_pile)
		{
			if ((*s_etat_processus).debug == d_vrai)
				if (((*s_etat_processus).type_debug &
						d_debug_pile_utilisateur) != 0)
			{
				if ((*s_etat_processus).langue == 'F')
				{
					printf("[%d] Dpilement de type %d "
							"(profondeur %lld)\n", (int) getpid(),
							(*(*s_objet)).type,
							(*s_etat_processus).hauteur_pile_operationnelle);
				}
				else
				{
					printf("[%d] Pulling a type %d object "
							"(depth %lld)\n", (int) getpid(),
							(*(*s_objet)).type,
							(*s_etat_processus).hauteur_pile_operationnelle);
				}

				fflush(stdout);
			}

			(*s_etat_processus).hauteur_pile_operationnelle--;
			estimation_taille_pile(s_etat_processus);
		}

		l_nouvelle_base_liste = (*l_ancienne_base_liste).suivant;

		*l_base_pile = l_nouvelle_base_liste;
		erreur = d_absence_erreur;

		if ((*s_etat_processus).taille_pile_tampon <= (10 * ((*s_etat_processus)
				.estimation_taille_pile_tampon + 1)))
		{
			// Enregistrement de la structure pour usage ultrieur.

			(*l_ancienne_base_liste).donnee = NULL;
			(*l_ancienne_base_liste).suivant = (*s_etat_processus).pile_tampon;
			(*s_etat_processus).pile_tampon = l_ancienne_base_liste;
			(*s_etat_processus).taille_pile_tampon++;
		}
		else
		{
			// Libration car le tampon est plein.

			free(l_ancienne_base_liste);
		}
	}

	return erreur;
}


/*
================================================================================
  Procdures affichant la pile oprationnelle
================================================================================
  Entre :
--------------------------------------------------------------------------------
  Sortie :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
affichage_pile(struct_processus *s_etat_processus, struct_liste_chainee
		*l_element_courant, integer8 niveau_courant)
{
	unsigned char			registre;

	registre = (*s_etat_processus).autorisation_conversion_chaine;
	(*s_etat_processus).autorisation_conversion_chaine = 'N';

	routine_recursive = 1;
	ecriture_pile(s_etat_processus, stdout, l_element_courant, niveau_courant);
	routine_recursive = 0;

	(*s_etat_processus).autorisation_conversion_chaine = registre;
	return;
}

// Bug de gcc  partir de gcc 4.6 (bug 48544)
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wclobbered"

void
ecriture_pile(struct_processus *s_etat_processus, file *flux,
		struct_liste_chainee *l, integer8 niveau_courant)
{
	unsigned char				*chaine;
	unsigned char				*registre;
	unsigned char				tampon[32 + 1];

	// vite le warning : argument l_element_courant might be clobbered by
	// longjmp or vfork
	volatile struct_liste_chainee	*l_element_courant;

	l_element_courant = l;

	if (l_element_courant != NULL)
	{
		if (setjmp(contexte_ecriture) == 0)
		{
			(*s_etat_processus).var_volatile_recursivite = -1;
			ecriture_pile(s_etat_processus, flux,
					(*l_element_courant).suivant, niveau_courant + 1);

			if ((*s_etat_processus).var_volatile_recursivite > 0)
			{
				(*s_etat_processus).var_volatile_recursivite--;

				if ((*s_etat_processus).var_volatile_recursivite == 0)
				{
					if (fprintf(flux, "%lld: ...\n", niveau_courant) < 0)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_erreur_fichier;
						return;
					}

					while(l_element_courant != NULL)
					{
						l_element_courant = (*l_element_courant).suivant;
					}
				}

				return;
			}

			(*s_etat_processus).var_volatile_recursivite = 0;
		}
		else
		{
			// Libration de n appels de la pile systme qui permet de
			// terminer la rcursion sans autre dpassement de pile.

			(*s_etat_processus).var_volatile_recursivite = 64;
			return;
		}

		sprintf(tampon, "%lld: ", niveau_courant);

		if ((chaine = formateur(s_etat_processus, (long) strlen(tampon),
				(*l_element_courant).donnee)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if ((*(*l_element_courant).donnee).type == CHN)
		{
			registre = chaine;

			if ((chaine = (unsigned char *) malloc((strlen(registre) + 3) *
					sizeof(unsigned char))) == NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			sprintf(chaine, "\"%s\"", registre);
			free(registre);
		}

		if (fprintf(flux, "%lld: %s\n", niveau_courant, chaine) < 0)
		{
			(*s_etat_processus).erreur_systeme = d_es_erreur_fichier;
			return;
		}

		free(chaine);
	}

	return;
}


/*
================================================================================
  Procdure imprimant la pile oprationnelle
================================================================================
  Entre : mthode 'C' = compacte, 'E' = tendue
--------------------------------------------------------------------------------
  Sortie :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
impression_pile(struct_processus *s_etat_processus,
		struct_liste_chainee *l, unsigned char methode, integer8 niveau_courant)
{
	struct_objet				s_objet;

	unsigned char				*chaine;
	unsigned char				*registre;
	unsigned char				tampon[32 + 1];

	// vite le warning : argument l_element_courant might be clobbered by
	// longjmp or vfork
	volatile struct_liste_chainee	*l_element_courant;

	l_element_courant = l;

	if (l_element_courant != NULL)
	{
		if (setjmp(contexte_impression) == 0)
		{
			(*s_etat_processus).var_volatile_recursivite = -1;
			impression_pile(s_etat_processus, (*l_element_courant).suivant,
					methode, niveau_courant + 1);

			if ((*s_etat_processus).var_volatile_recursivite > 0)
			{
				(*s_etat_processus).var_volatile_recursivite--;

				if ((*s_etat_processus).var_volatile_recursivite == 0)
				{
					while(l_element_courant != NULL)
					{
						l_element_courant = (*l_element_courant).suivant;
					}
				}

				return;
			}

			(*s_etat_processus).var_volatile_recursivite = 0;
		}
		else
		{
			(*s_etat_processus).var_volatile_recursivite = 16;
			return;
		}

		if (methode == 'C')
		{
			s_objet.type = CHN;
			sprintf(tampon, "%lld: ", niveau_courant);

			if ((chaine = formateur(s_etat_processus, (long) strlen(tampon),
					(*l_element_courant).donnee)) == NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			if ((*(*l_element_courant).donnee).type == CHN)
			{
				registre = chaine;

				if ((chaine = (unsigned char *) malloc((strlen(registre) + 3) *
						sizeof(unsigned char))) == NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				sprintf(chaine, "\"%s\"", registre);
				free(registre);
			}

			if ((s_objet.objet = malloc((strlen(chaine) + 64) *
					sizeof(unsigned char))) == NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			sprintf((unsigned char *) s_objet.objet,
					"\n\\noindent\\begin{verbatim}\n%lld: %s\n\\end{verbatim}",
					niveau_courant, chaine);
			free(chaine);

			formateur_tex(s_etat_processus, &s_objet, 'V');
			free(s_objet.objet);
		}
		else
		{
			formateur_tex(s_etat_processus, (*l_element_courant).donnee, 'N');
		}
	}

	return;
}

#pragma GCC diagnostic pop

// vim: ts=4
