/*
================================================================================
  RPL/2 (R) version 4.1.21
  Copyright (C) 1989-2015 Dr. BERTRAND Jol

  This file is part of RPL/2.

  RPL/2 is free software; you can redistribute it and/or modify it
  under the terms of the CeCILL V2 License as published by the french
  CEA, CNRS and INRIA.
 
  RPL/2 is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the CeCILL V2 License
  for more details.
 
  You should have received a copy of the CeCILL License
  along with RPL/2. If not, write to info@cecill.info.
================================================================================
*/


#include "rpl-conv.h"


/*
================================================================================
  Procdure d'empilement des arguments d'une commande dans la pile LAST
================================================================================
  Entre : structure processus et nombre d'aguments  empiler
--------------------------------------------------------------------------------
  Sortie : drapeau d'erreur
--------------------------------------------------------------------------------
  Effets de bord : efface le prcdent contenu de la pile LAST
================================================================================
*/

logical1
empilement_pile_last(struct_processus *s_etat_processus,
		integer8 nombre_arguments)
{
	struct_liste_chainee			*l_element_courant;
	struct_liste_chainee			*l_element_suivant;

	struct_objet					*s_objet;

	logical1						erreur;

	integer8						i;

	erreur = d_absence_erreur;
	l_element_courant = (*s_etat_processus).l_base_pile_last;

	while(l_element_courant != NULL)
	{
		liberation(s_etat_processus, (*l_element_courant).donnee);
		l_element_suivant = (*l_element_courant).suivant;

		// On ne libre pas le maillon de la chane. On le sauvegarde
		// arbitrairement dans le tampon.

		(*l_element_courant).donnee = NULL;
		(*l_element_courant).suivant = (*s_etat_processus).pile_tampon;
		(*s_etat_processus).pile_tampon = l_element_courant;
		(*s_etat_processus).taille_pile_tampon++;

		l_element_courant = l_element_suivant;
	}

	(*s_etat_processus).l_base_pile_last = NULL;
	l_element_courant = (*s_etat_processus).l_base_pile;

	for(i = 0; ((i < nombre_arguments) && (erreur == d_absence_erreur)
			&& (l_element_courant != NULL)); i++)
	{
		s_objet = copie_objet(s_etat_processus,
				(*l_element_courant).donnee, 'P');

		if (s_objet != NULL)
		{
			erreur = empilement(s_etat_processus,
					&((*s_etat_processus).l_base_pile_last), s_objet);
			l_element_courant = (*l_element_courant).suivant;
		}
		else
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			break;
		}
	}

	if (i != nombre_arguments)
	{
		/*
		 * Ne renvoie pas d'erreur si le nombre d'argument ne correspond pas
		 *  celui attendu pour pouvoir traiter correctement les fuites
		 * mmoire dans le traitement de la fonction IFERR. Le traitement
		 * du nombre d'arguments est fait au niveau de la fonction appelant
		 * "empilement_pile_last".
		 */

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;

		/*
		 * En cas de nombre d'arguments insuffisant, cette routine
		 * dclenche une erreur. Si la pile LAST est active, la rcupration
		 * de l'erreur force la copie de la pile LAST dans la pile
		 * oprationnelle courante. Dans notre cas, les arguments n'ont pas
		 * encore t consomms pas la routine appelante et il convient
		 * d'liminer les arguments de la pile oprationnelle avant de retourner
		 *  la routine appelante.
		 */

		l_element_courant = (*s_etat_processus).l_base_pile;

		for(; i > 0; i--)
		{
			if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
					&s_objet) == d_erreur)
			{
				(*s_etat_processus).erreur_execution = d_ex_manque_argument;
				return(erreur);
			}

			liberation(s_etat_processus, s_objet);
		}
	}

	return(erreur);
}

// vim: ts=4
