/*
================================================================================
  RPL/2 (R) version 4.1.21
  Copyright (C) 1989-2015 Dr. BERTRAND Jol

  This file is part of RPL/2.

  RPL/2 is free software; you can redistribute it and/or modify it
  under the terms of the CeCILL V2 License as published by the french
  CEA, CNRS and INRIA.
 
  RPL/2 is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the CeCILL V2 License
  for more details.
 
  You should have received a copy of the CeCILL License
  along with RPL/2. If not, write to info@cecill.info.
================================================================================
*/


#include "rpl-conv.h"


/*
================================================================================
  Fonction 'array->'
================================================================================
  Entres : pointeur sur une structure struct_processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_array_fleche(struct_processus *s_etat_processus)
{
	integer8						i;
	integer8						j;

	struct_liste_chainee			*l_element_courant;

	struct_objet					*s_objet_source;
	struct_objet					*s_objet_elementaire;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  ARRAY-> [ARRY->] ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(clatement de vecteur ou de matrice)\n\n");
		}
		else
		{
			printf("(vector or matrix split)\n\n");
		}

		printf("    1: %s\n", d_VIN);
		printf("->  n: %s\n", d_INT);
		printf("    ...\n");
		printf("    1: %s\n\n", d_INT);

		printf("    1: %s\n", d_VRL);
		printf("->  n: %s\n", d_REL);
		printf("    ...\n");
		printf("    1: %s\n\n", d_REL);

		printf("    1: %s\n", d_VCX);
		printf("->  n: %s\n", d_CPL);
		printf("    ...\n");
		printf("    1: %s\n\n", d_CPL);

		printf("    1: %s\n", d_MIN);
		printf("-> nm: %s\n", d_INT);
		printf("    ...\n");
		printf("    1: %s\n\n", d_INT);

		printf("    1: %s\n", d_MRL);
		printf("-> nm: %s\n", d_REL);
		printf("    ...\n");
		printf("    1: %s\n\n", d_REL);

		printf("    1: %s\n", d_MCX);
		printf("-> nm: %s\n", d_CPL);
		printf("    ...\n");
		printf("    1: %s\n", d_CPL);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 1) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_source) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

/*
--------------------------------------------------------------------------------
  Cas des vecteurs
--------------------------------------------------------------------------------
*/

	if ((*s_objet_source).type == VIN)
	{
		/*
		 * Traitement d'un vecteur d'entiers
		 */

		for(i = 0; i < (*((struct_vecteur *) (*s_objet_source).objet)).taille;
				i++)
		{
			if ((s_objet_elementaire = allocation(s_etat_processus, INT))
					== NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			(*((integer8 *) (*s_objet_elementaire).objet)) =
					((integer8 *) (*((struct_vecteur *)
					(*s_objet_source).objet)).tableau)[i];

			if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
					s_objet_elementaire) == d_erreur)
			{
				return;
			}
		}

		if ((s_objet_elementaire = allocation(s_etat_processus, LST))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*s_objet_elementaire).objet =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (struct_liste_chainee *)
				(*s_objet_elementaire).objet;

		(*l_element_courant).suivant = NULL;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, INT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((integer8 *) (*(*l_element_courant).donnee).objet)) =
				(*((struct_vecteur *) (*s_objet_source).objet)).taille;
		
		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_elementaire) == d_erreur)
		{
			return;
		}
	}
	else if ((*s_objet_source).type == VRL)
	{
		/*
		 * Traitement d'un vecteur de rels
		 */

		for(i = 0; i < (*((struct_vecteur *) (*s_objet_source).objet)).taille;
				i++)
		{
			if ((s_objet_elementaire = allocation(s_etat_processus, REL))
					== NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			(*((real8 *) (*s_objet_elementaire).objet)) =
					((real8 *) (*((struct_vecteur *)
					(*s_objet_source).objet)).tableau)[i];

			if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
					s_objet_elementaire) == d_erreur)
			{
				return;
			}
		}

		if ((s_objet_elementaire = allocation(s_etat_processus, LST))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*s_objet_elementaire).objet =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (struct_liste_chainee *)
				(*s_objet_elementaire).objet;

		(*l_element_courant).suivant = NULL;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, INT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((integer8 *) (*(*l_element_courant).donnee).objet)) =
				(*((struct_vecteur *) (*s_objet_source).objet)).taille;
		
		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_elementaire) == d_erreur)
		{
			return;
		}
	}
	else if ((*s_objet_source).type == VCX)
	{
		/*
		 * Traitement d'un vecteur de complexes
		 */

		for(i = 0; i < (*((struct_vecteur *) (*s_objet_source).objet)).taille;
				i++)
		{
			if ((s_objet_elementaire = allocation(s_etat_processus, CPL))
					== NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			(*((struct_complexe16 *) (*s_objet_elementaire).objet))
					.partie_reelle = ((struct_complexe16 *)
					(*((struct_vecteur *) (*s_objet_source).objet)).tableau)[i]
					.partie_reelle;
			(*((struct_complexe16 *) (*s_objet_elementaire).objet))
					.partie_imaginaire = ((struct_complexe16 *)
					(*((struct_vecteur *) (*s_objet_source).objet)).tableau)[i]
					.partie_imaginaire;

			if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
					s_objet_elementaire) == d_erreur)
			{
				return;
			}
		}

		if ((s_objet_elementaire = allocation(s_etat_processus, LST))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*s_objet_elementaire).objet =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (struct_liste_chainee *)
				(*s_objet_elementaire).objet;

		(*l_element_courant).suivant = NULL;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, INT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((integer8 *) (*(*l_element_courant).donnee).objet)) =
				(*((struct_vecteur *) (*s_objet_source).objet)).taille;
		
		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_elementaire) == d_erreur)
		{
			return;
		}
	}

/*
--------------------------------------------------------------------------------
  Cas des matrices
--------------------------------------------------------------------------------
*/

	else if ((*s_objet_source).type == MIN)
	{
		/*
		 * Traitement d'une matrice d'entiers
		 */

		for(i = 0; i < (*((struct_matrice *) (*s_objet_source).objet))
				.nombre_lignes; i++)
		{
			for(j = 0; j < (*((struct_matrice *) (*s_objet_source).objet))
					.nombre_colonnes; j++)
			{
				if ((s_objet_elementaire = allocation(s_etat_processus, INT))
						== NULL)
				{
					(*s_etat_processus).erreur_systeme =
					   		d_es_allocation_memoire;
					return;
				}

				(*((integer8 *) (*s_objet_elementaire).objet)) =
						((integer8 **) (*((struct_matrice *)
						(*s_objet_source).objet)).tableau)[i][j];

				if (empilement(s_etat_processus, &((*s_etat_processus)
						.l_base_pile), s_objet_elementaire) == d_erreur)
				{
					return;
				}
			}
		}

		if ((s_objet_elementaire = allocation(s_etat_processus, LST))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*s_objet_elementaire).objet =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (struct_liste_chainee *)
				(*s_objet_elementaire).objet;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, INT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((integer8 *) (*(*l_element_courant).donnee).objet)) =
				(*((struct_matrice *) (*s_objet_source).objet))
				.nombre_lignes;
		
		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;
		(*l_element_courant).suivant = NULL;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, INT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((integer8 *) (*(*l_element_courant).donnee).objet)) =
				(*((struct_matrice *) (*s_objet_source).objet))
				.nombre_colonnes;

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_elementaire) == d_erreur)
		{
			return;
		}
	}
	else if ((*s_objet_source).type == MRL)
	{
		/*
		 * Traitement d'une matrice de rels
		 */

		for(i = 0; i < (*((struct_matrice *) (*s_objet_source).objet))
				.nombre_lignes; i++)
		{
			for(j = 0; j < (*((struct_matrice *) (*s_objet_source).objet))
					.nombre_colonnes; j++)
			{
				if ((s_objet_elementaire = allocation(s_etat_processus, REL))
						== NULL)
				{
					(*s_etat_processus).erreur_systeme =
					   		d_es_allocation_memoire;
					return;
				}

				(*((real8 *) (*s_objet_elementaire).objet)) =
						((real8 **) (*((struct_matrice *)
						(*s_objet_source).objet)).tableau)[i][j];

				if (empilement(s_etat_processus, &((*s_etat_processus)
						.l_base_pile), s_objet_elementaire) == d_erreur)
				{
					return;
				}
			}
		}

		if ((s_objet_elementaire = allocation(s_etat_processus, LST))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*s_objet_elementaire).objet =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (struct_liste_chainee *)
				(*s_objet_elementaire).objet;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, INT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((integer8 *) (*(*l_element_courant).donnee).objet)) =
				(*((struct_matrice *) (*s_objet_source).objet))
				.nombre_lignes;
		
		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;
		(*l_element_courant).suivant = NULL;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, INT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((integer8 *) (*(*l_element_courant).donnee).objet)) =
				(*((struct_matrice *) (*s_objet_source).objet))
				.nombre_colonnes;

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_elementaire) == d_erreur)
		{
			return;
		}
	}
	else if ((*s_objet_source).type == MCX)
	{
		/*
		 * Traitement d'une matrice de complexes
		 */

		for(i = 0; i < (*((struct_matrice *) (*s_objet_source).objet))
				.nombre_lignes; i++)
		{
			for(j = 0; j < (*((struct_matrice *) (*s_objet_source).objet))
					.nombre_colonnes; j++)
			{
				if ((s_objet_elementaire = allocation(s_etat_processus, CPL))
						== NULL)
				{
					(*s_etat_processus).erreur_systeme =
					   		d_es_allocation_memoire;
					return;
				}

				(*((struct_complexe16 *) (*s_objet_elementaire).objet))
				   		.partie_reelle = ((struct_complexe16 **)
						(*((struct_matrice *) (*s_objet_source).objet))
						.tableau)[i][j].partie_reelle;
				(*((struct_complexe16 *) (*s_objet_elementaire).objet))
				   		.partie_imaginaire = ((struct_complexe16 **)
						(*((struct_matrice *) (*s_objet_source).objet))
						.tableau)[i][j].partie_imaginaire;

				if (empilement(s_etat_processus, &((*s_etat_processus)
						.l_base_pile), s_objet_elementaire) == d_erreur)
				{
					return;
				}
			}
		}

		if ((s_objet_elementaire = allocation(s_etat_processus, LST))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*s_objet_elementaire).objet =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (struct_liste_chainee *)
				(*s_objet_elementaire).objet;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, INT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((integer8 *) (*(*l_element_courant).donnee).objet)) =
				(*((struct_matrice *) (*s_objet_source).objet))
				.nombre_lignes;
		
		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;
		(*l_element_courant).suivant = NULL;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, INT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((integer8 *) (*(*l_element_courant).donnee).objet)) =
				(*((struct_matrice *) (*s_objet_source).objet))
				.nombre_colonnes;

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_elementaire) == d_erreur)
		{
			return;
		}
	}

/*
--------------------------------------------------------------------------------
  Ralisation impossible de la fonction ARRAY->
--------------------------------------------------------------------------------
*/

	else
	{
		liberation(s_etat_processus, s_objet_source);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_source);

	return;
}


/*
================================================================================
  Fonction 'alog'
================================================================================
  Entres : pointeur sur une struct_processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_alog(struct_processus *s_etat_processus)
{
	integer8						base;
	integer8						tampon;

	struct_liste_chainee			*l_element_courant;
	struct_liste_chainee			*l_element_precedent;

	struct_objet					*s_copie_argument;
	struct_objet					*s_objet_argument;
	struct_objet					*s_objet_resultat;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  ALOG ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(antilogarithme base 10)\n\n");
		}
		else
		{
			printf("(10-based antilogarithm)\n\n");
		}

		printf("    1: %s\n", d_INT);
		printf("->  1: %s, %s\n\n", d_INT, d_REL);

		printf("    1: %s\n", d_REL);
		printf("->  1: %s\n", d_REL);

		printf("    1: %s\n", d_CPL);
		printf("->  1: %s\n", d_CPL);

		printf("    1: %s, %s\n", d_NOM, d_ALG);
		printf("->  1: %s\n\n", d_ALG);

		printf("    1: %s\n", d_RPN);
		printf("->  1: %s\n", d_RPN);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = 1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 1) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

/*
--------------------------------------------------------------------------------
  Alog d'un entier
--------------------------------------------------------------------------------
*/

	if ((*s_objet_argument).type == INT)
	{
		base = 10;

		if (depassement_puissance(&base, (integer8 *) (*s_objet_argument).objet,
				&tampon) == d_absence_erreur)
		{
			if ((s_objet_resultat = allocation(s_etat_processus, INT))
					== NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			(*((integer8 *) (*s_objet_resultat).objet)) = tampon;
		}
		else
		{
			if ((s_objet_resultat = allocation(s_etat_processus, REL))
					== NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			(*((real8 *) (*s_objet_resultat).objet)) =
					pow((real8) 10, (real8) (*((integer8 *)
					(*s_objet_argument).objet)));
		}
	}

/*
--------------------------------------------------------------------------------
  Alog d'un rel
--------------------------------------------------------------------------------
*/

	else if ((*s_objet_argument).type == REL)
	{
		if ((s_objet_resultat = allocation(s_etat_processus, REL))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((real8 *) (*s_objet_resultat).objet)) =
				pow((real8) 10, ((*((real8 *) (*s_objet_argument).objet))));
	}

/*
--------------------------------------------------------------------------------
  Alog d'un complexe
--------------------------------------------------------------------------------
*/

	else if ((*s_objet_argument).type == CPL)
	{
		if ((s_objet_resultat = allocation(s_etat_processus, CPL))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		f77alogc_(&((*((struct_complexe16 *) (*s_objet_argument).objet))),
				(struct_complexe16 *) (*s_objet_resultat).objet);
	}

/*
--------------------------------------------------------------------------------
  Alog d'un nom
--------------------------------------------------------------------------------
*/

	else if ((*s_objet_argument).type == NOM)
	{
		if ((s_objet_resultat = allocation(s_etat_processus, ALG))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*s_objet_resultat).objet =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*s_objet_resultat).objet;

		if (((*l_element_courant).donnee =
				allocation(s_etat_processus, FCT)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 0;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_alog;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(3 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, "<<");

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;
		(*l_element_courant).donnee = s_objet_argument;

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;

		if (((*l_element_courant).donnee =
				allocation(s_etat_processus, FCT)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 1;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_alog;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(5 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}
			
		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, "ALOG");

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;

		if (((*l_element_courant).donnee =
				allocation(s_etat_processus, FCT)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 0;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_vers_niveau_inferieur;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(3 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, ">>");

		(*l_element_courant).suivant = NULL;
		s_objet_argument = NULL;
	}

/*
--------------------------------------------------------------------------------
  Alog d'une expression
--------------------------------------------------------------------------------
*/

	else if (((*s_objet_argument).type == ALG) ||
			((*s_objet_argument).type == RPN))
	{
		if ((s_copie_argument = copie_objet(s_etat_processus,
				s_objet_argument, 'N')) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (struct_liste_chainee *)
				(*s_copie_argument).objet;
		l_element_precedent = l_element_courant;

		while((*l_element_courant).suivant != NULL)
		{
			l_element_precedent = l_element_courant;
			l_element_courant = (*l_element_courant).suivant;
		}

		if (((*l_element_precedent).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*(*l_element_precedent).suivant).donnee =
				allocation(s_etat_processus, FCT)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*(*l_element_precedent).suivant)
				.donnee).objet)).nombre_arguments = 1;
		(*((struct_fonction *) (*(*(*l_element_precedent).suivant)
				.donnee).objet)).fonction = instruction_alog;

		if (((*((struct_fonction *) (*(*(*l_element_precedent)
				.suivant).donnee).objet)).nom_fonction =
				malloc(5 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*(*l_element_precedent)
				.suivant).donnee).objet)).nom_fonction, "ALOG");

		(*(*l_element_precedent).suivant).suivant = l_element_courant;

		s_objet_resultat = s_copie_argument;
	}

/*
--------------------------------------------------------------------------------
  Fonction alog impossible  raliser
--------------------------------------------------------------------------------
*/

	else
	{
		liberation(s_etat_processus, s_objet_argument);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument);

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_resultat) == d_erreur)
	{
		return;
	}

	return;
}

// vim: ts=4
