/*
================================================================================
  RPL/2 (R) version 4.1.21
  Copyright (C) 1989-2015 Dr. BERTRAND Jol

  This file is part of RPL/2.

  RPL/2 is free software; you can redistribute it and/or modify it
  under the terms of the CeCILL V2 License as published by the french
  CEA, CNRS and INRIA.
 
  RPL/2 is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the CeCILL V2 License
  for more details.
 
  You should have received a copy of the CeCILL License
  along with RPL/2. If not, write to info@cecill.info.
================================================================================
*/


#include "rpl-conv.h"


/*
================================================================================
  Fonction 'in'
================================================================================
  Entres :
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_in(struct_processus *s_etat_processus)
{
	logical1				difference;

	struct_liste_chainee	*l_element_courant;

	struct_objet			*s_copie_1;
	struct_objet			*s_copie_2;
	struct_objet			*s_objet_argument_1;
	struct_objet			*s_objet_argument_2;
	struct_objet			*s_objet_resultat;
	struct_objet			*s_objet_resultat_intermediaire;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  IN ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(test de l'appartenance  un ensemble)\n\n");
		}
		else
		{
			printf("(check membership)\n\n");
		}

		printf("    2: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN);
		printf("    1: %s\n", d_LST);
		printf("->  1: %s\n", d_INT);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 2) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_1) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_2) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_argument_1);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (((*s_objet_argument_1).type == LST) &&
			(((*s_objet_argument_2).type == INT) ||
			((*s_objet_argument_2).type == REL) ||
			((*s_objet_argument_2).type == CPL) ||
			((*s_objet_argument_2).type == VIN) ||
			((*s_objet_argument_2).type == VRL) ||
			((*s_objet_argument_2).type == VCX) ||
			((*s_objet_argument_2).type == MIN) ||
			((*s_objet_argument_2).type == MRL) ||
			((*s_objet_argument_2).type == MCX) ||
			((*s_objet_argument_2).type == TAB) ||
			((*s_objet_argument_2).type == BIN) ||
			((*s_objet_argument_2).type == NOM) ||
			((*s_objet_argument_2).type == CHN) ||
			((*s_objet_argument_2).type == LST) ||
			((*s_objet_argument_2).type == ALG) ||
			((*s_objet_argument_2).type == RPN)))
	{
		if ((s_objet_resultat = allocation(s_etat_processus, INT)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (struct_liste_chainee *)
				(*s_objet_argument_1).objet;

		difference = d_vrai;

		while((difference == d_vrai) && (l_element_courant != NULL))
		{
			if ((s_copie_1 = copie_objet(s_etat_processus,
					(*l_element_courant).donnee, 'P')) == NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
					s_copie_1) == d_erreur)
			{
				return;
			}

			if ((s_copie_2 = copie_objet(s_etat_processus,
					s_objet_argument_2, 'P')) == NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
					s_copie_2) == d_erreur)
			{
				return;
			}

			instruction_same(s_etat_processus);

			if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
					 &s_objet_resultat_intermediaire) == d_erreur)
			{
				liberation(s_etat_processus, s_objet_argument_1);
				liberation(s_etat_processus, s_objet_argument_2);
				liberation(s_etat_processus, s_objet_resultat);

				(*s_etat_processus).erreur_execution = d_ex_manque_argument;
				return;
			}

			if ((*s_objet_resultat_intermediaire).type != INT)
			{
				liberation(s_etat_processus, s_objet_resultat_intermediaire);
				liberation(s_etat_processus, s_objet_argument_1);
				liberation(s_etat_processus, s_objet_argument_2);
				liberation(s_etat_processus, s_objet_resultat);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}

			difference = (*(((integer8 *) (*s_objet_resultat_intermediaire)
					.objet)) == 0) ? d_vrai : d_faux;

			liberation(s_etat_processus, s_objet_resultat_intermediaire);
			l_element_courant = (*l_element_courant).suivant;
		}

		if (difference == d_vrai)
		{
			(*((integer8 *) (*s_objet_resultat).objet)) = 0;
		}
		else
		{
			(*((integer8 *) (*s_objet_resultat).objet)) = -1;
		}
	}
	else
	{
		liberation(s_etat_processus, s_objet_argument_1);
		liberation(s_etat_processus, s_objet_argument_2);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument_1);
	liberation(s_etat_processus, s_objet_argument_2);

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_resultat) == d_erreur)
	{
		return;
	}

	return;
}


/*
================================================================================
  Fonction 'implicit'
================================================================================
  Entres :
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_implicit(struct_processus *s_etat_processus)
{
	struct_objet				*s_objet_argument;

	unsigned char				*commande;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  IMPLICIT ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(gestion des noms implicites)\n\n");
		}
		else
		{
			printf("(implicit names management)\n\n");
		}

		printf("    1: %s\n\n", d_CHN);

		if ((*s_etat_processus).langue == 'F')
		{
			printf("  Utilisation :\n\n");
		}
		else
		{
			printf("  Usage:\n\n");
		}

		printf("    \"NONE\" IMPLICIT\n");
		printf("    \"ALL\" IMPLICIT\n");
		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 1) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if ((*s_objet_argument).type == CHN)
	{
		if ((commande = conversion_majuscule(s_etat_processus, (unsigned char *)
				(*s_objet_argument).objet)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (strcmp(commande, "NONE") == 0)
		{
			(*s_etat_processus).autorisation_nom_implicite = 'N';
		}
		else if (strcmp(commande, "ALL") == 0)
		{
			(*s_etat_processus).autorisation_nom_implicite = 'Y';
		}
		else
		{
			(*s_etat_processus).erreur_execution = d_ex_argument_invalide;
		}

		free(commande);
	}
	else
	{
		liberation(s_etat_processus, s_objet_argument);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument);
	return;
}


/*
================================================================================
  Fonction 'infinity'
================================================================================
  Entres :
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_sensible_infinity(struct_processus *s_etat_processus)
{
	if (strcmp((*s_etat_processus).instruction_courante, "infinity") == 0)
	{
		instruction_infinity(s_etat_processus);
	}
	else
	{
		(*s_etat_processus).instruction_valide = 'N';
	}

	return;
}

void
instruction_infinity(struct_processus *s_etat_processus)
{
	struct_objet				*s_objet;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  infinity ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(infini)\n\n");
		}
		else
		{
			printf("(infinity constant)\n\n");
		}

		printf("->  1: %s\n", d_REL);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).constante_symbolique = 'Y';
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	/* Indicateur 35 arm => valuation symbolique */
	if (test_cfsf(s_etat_processus, 35) == d_vrai)
	{
		if ((s_objet = allocation(s_etat_processus, NOM)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*((struct_nom *) (*s_objet).objet)).nom =
				malloc(9 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_nom *) (*s_objet).objet)).nom, "infinity");
		(*((struct_nom *) (*s_objet).objet)).symbole = d_faux;
	}
	else
	{
		if ((s_objet = allocation(s_etat_processus, REL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

#		ifdef FP_INFINITE
		(*((real8 *) (*s_objet).objet)) = (double) INFINITY;
#		else
		(*((real8 *) (*s_objet).objet)) = HUGE_VAL;
#		endif
	}

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet) == d_erreur)
	{
		return;
	}

	return;
}

// vim: ts=4
