/*
================================================================================
  RPL/2 (R) version 4.1.21
  Copyright (C) 1989-2015 Dr. BERTRAND Jol

  This file is part of RPL/2.

  RPL/2 is free software; you can redistribute it and/or modify it
  under the terms of the CeCILL V2 License as published by the french
  CEA, CNRS and INRIA.
 
  RPL/2 is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the CeCILL V2 License
  for more details.
 
  You should have received a copy of the CeCILL License
  along with RPL/2. If not, write to info@cecill.info.
================================================================================
*/


#include "rpl-conv.h"


/*
================================================================================
  Fonction 'p->r'
================================================================================
  Entres : pointeur sur une structure struct_processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_p_vers_r(struct_processus *s_etat_processus)
{
	double							angle;

	struct_liste_chainee			*l_element_courant;
	struct_liste_chainee			*l_element_precedent;

	struct_objet					*s_copie_argument;
	struct_objet					*s_objet_argument;
	struct_objet					*s_objet_resultat;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  P->R ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(coordonnes polaires vers cartsiennes)\n\n");
		}
		else
		{
			printf("(polar to cartesian coordinates)\n\n");
		}

		printf("    1: %s, %s, %s\n", d_INT, d_REL, d_CPL);
		printf("->  1: %s\n\n", d_CPL);

		printf("    1: %s, %s\n", d_NOM, d_ALG);
		printf("->  1: %s\n\n", d_ALG);

		printf("    1: %s\n", d_RPN);
		printf("->  1: %s\n", d_RPN);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 1) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

/*
--------------------------------------------------------------------------------
  Conversion d'un entier ou d'un rel
--------------------------------------------------------------------------------
*/

	if (((*s_objet_argument).type == INT) ||
			((*s_objet_argument).type == REL))
	{
		if ((s_objet_resultat = allocation(s_etat_processus, CPL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if ((*s_objet_argument).type == INT)
		{
			(*((struct_complexe16 *) (*s_objet_resultat).objet)).partie_reelle
					= (real8) (*((integer8 *) (*s_objet_argument).objet));
		}
		else
		{
			(*((struct_complexe16 *) (*s_objet_resultat).objet)).partie_reelle
					= (*((real8 *) (*s_objet_argument).objet));
		}

		(*((struct_complexe16 *) (*s_objet_resultat).objet)).partie_imaginaire
				= 0;
	}

/*
--------------------------------------------------------------------------------
  Conversion d'un complexe
--------------------------------------------------------------------------------
*/

	else if ((*s_objet_argument).type == CPL)
	{
		if ((s_objet_resultat = allocation(s_etat_processus, CPL)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		angle = (*((struct_complexe16 *) (*s_objet_argument).objet))
				.partie_imaginaire;

		if (test_cfsf(s_etat_processus, 60) == d_faux)
		{
			conversion_degres_vers_radians(&angle);
		}

		(*((struct_complexe16 *) (*s_objet_resultat).objet)).partie_reelle =
				(*((struct_complexe16 *) (*s_objet_argument).objet))
				.partie_reelle * cos(angle);
		(*((struct_complexe16 *) (*s_objet_resultat).objet)).partie_imaginaire =
				(*((struct_complexe16 *) (*s_objet_argument).objet))
				.partie_reelle * sin(angle);
	}

/*
--------------------------------------------------------------------------------
  Conversion d'un nom
--------------------------------------------------------------------------------
*/

	else if ((*s_objet_argument).type == NOM)
	{
		if ((s_objet_resultat = allocation(s_etat_processus, ALG)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*s_objet_resultat).objet =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*s_objet_resultat).objet;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, FCT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 0;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_vers_niveau_superieur;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(3 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, "<<");

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;
		(*l_element_courant).donnee = s_objet_argument;

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, FCT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 1;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_p_vers_r;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(5 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, "P->R");

		if (((*l_element_courant).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (*l_element_courant).suivant;

		if (((*l_element_courant).donnee = allocation(s_etat_processus, FCT))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nombre_arguments = 0;
		(*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.fonction = instruction_vers_niveau_inferieur;

		if (((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction = malloc(3 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*l_element_courant).donnee).objet))
				.nom_fonction, ">>");

		(*l_element_courant).suivant = NULL;
		s_objet_argument = NULL;
	}

/*
--------------------------------------------------------------------------------
  Conversion d'une expression
--------------------------------------------------------------------------------
*/

	else if (((*s_objet_argument).type == ALG) ||
			((*s_objet_argument).type == RPN))
	{
		if ((s_copie_argument = copie_objet(s_etat_processus, s_objet_argument,
				'N')) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant = (struct_liste_chainee *)
				(*s_copie_argument).objet;
		l_element_precedent = l_element_courant;

		while((*l_element_courant).suivant != NULL)
		{
			l_element_precedent = l_element_courant;
			l_element_courant = (*l_element_courant).suivant;
		}

		if (((*l_element_precedent).suivant =
				allocation_maillon(s_etat_processus)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*(*l_element_precedent).suivant).donnee =
				allocation(s_etat_processus, FCT)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		(*((struct_fonction *) (*(*(*l_element_precedent).suivant)
				.donnee).objet)).nombre_arguments = 1;
		(*((struct_fonction *) (*(*(*l_element_precedent).suivant)
				.donnee).objet)).fonction = instruction_p_vers_r;

		if (((*((struct_fonction *) (*(*(*l_element_precedent)
				.suivant).donnee).objet)).nom_fonction =
				malloc(5 * sizeof(unsigned char))) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		strcpy((*((struct_fonction *) (*(*(*l_element_precedent)
				.suivant).donnee).objet)).nom_fonction, "P->R");

		(*(*l_element_precedent).suivant).suivant = l_element_courant;

		s_objet_resultat = s_copie_argument;
	}

/*
--------------------------------------------------------------------------------
  Ralisation impossible de la fonction P->R
--------------------------------------------------------------------------------
*/

	else
	{
		liberation(s_etat_processus, s_objet_argument);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument);

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_resultat) == d_erreur)
	{
		return;
	}

	return;
}


/*
================================================================================
  Fonction 'put'
================================================================================
  Entres : pointeur sur une structure struct_processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_put(struct_processus *s_etat_processus)
{
	logical1							variable_partagee;

	struct_liste_chainee				*l_element_courant;

	struct_objet						*s_copie_3;
	struct_objet						*s_copie_4;
	struct_objet						*s_objet_1;
	struct_objet						*s_objet_2;
	struct_objet						*s_objet_3;
	struct_objet						*s_objet_4;
	struct_objet						*s_objet_element;

	integer8							i;
	integer8							indice_i;
	integer8							indice_j;
	integer8							j;
	integer8							nombre_dimensions;

	void								*tampon;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  PUT ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(change un lment)\n\n");
		}
		else
		{
			printf("(change element)\n\n");
		}

		printf("    3: %s, %s, %s\n", d_VIN, d_VRL, d_VCX);
		printf("    2: %s\n", d_LST);
		printf("    1: %s, %s, %s\n", d_INT, d_REL, d_CPL);
		printf("->  1: %s, %s, %s\n\n", d_VIN, d_VRL, d_VCX);

		printf("    3: %s, %s, %s\n", d_MIN, d_MRL, d_MCX);
		printf("    2: %s\n", d_LST);
		printf("    1: %s, %s, %s\n", d_INT, d_REL, d_CPL);
		printf("->  1: %s, %s, %s\n\n", d_MIN, d_MRL, d_MCX);

		printf("    3: %s\n", d_LST);
		printf("    2: %s\n", d_INT);
		printf("    1: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);
		printf("->  1: %s\n\n", d_LST);

		printf("    3: %s\n", d_TAB);
		printf("    2: %s\n", d_LST);
		printf("    1: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);
		printf("->  1: %s\n\n", d_TAB);

		printf("    3: %s\n", d_NOM);
		printf("    2: %s, %s\n", d_LST, d_INT);
		printf("    1: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 3) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_1) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_2) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_1);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_3) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_1);
		liberation(s_etat_processus, s_objet_2);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

/*
--------------------------------------------------------------------------------
  Traitement des lments des vecteurs
--------------------------------------------------------------------------------
*/

	if (((*s_objet_3).type == VIN) ||
			((*s_objet_3).type == VRL) ||
			((*s_objet_3).type == VCX))
	{
		if ((*s_objet_2).type != LST)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
			return;
		}

		l_element_courant = (*s_objet_2).objet;
		nombre_dimensions = 0;

		while(l_element_courant != NULL)
		{
			nombre_dimensions++;
			l_element_courant = (*l_element_courant).suivant;
		}

		if (nombre_dimensions != 1)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		l_element_courant = (*s_objet_2).objet;

		if ((*(*l_element_courant).donnee).type != INT)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
			return;
		}

		if ((*((integer8 *) (*(*l_element_courant).donnee).objet)) <= 0)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_argument_invalide;
			return;
		}
		else if ((*((integer8 *) (*(*l_element_courant).donnee).objet)) >
				(integer8) (*((struct_vecteur *) (*s_objet_3).objet)).taille)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
			return;
		}

		indice_i = (*((integer8 *) (*(*l_element_courant).donnee).objet));

		if ((s_copie_3 = copie_objet(s_etat_processus, s_objet_3, 'Q')) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		liberation(s_etat_processus, s_objet_3);
		s_objet_3 = s_copie_3;

		if ((*s_objet_3).type == VIN)
		{
			/*
			 * Vecteur d'entiers
			 */

			if ((*s_objet_1).type == INT)
			{
				/*
				 * Aucune conversion de type
				 */

				((integer8 *) (*((struct_vecteur *) (*s_objet_3).objet))
						.tableau)[indice_i - 1] = (*((integer8 *)
						(*s_objet_1).objet));
			}
			else if ((*s_objet_1).type == REL)
			{
				/*
				 * Conversion du vecteur en vecteur rel
				 */

				tampon = (void *) ((integer8 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau);

				(*((struct_vecteur *) (*s_objet_3).objet)).type = 'R';
				(*s_objet_3).type = VRL;

				if (((*((struct_vecteur *) (*s_objet_3).objet)).tableau
						= malloc(((size_t) (*((struct_vecteur *)
						(*s_objet_3).objet)).taille) * sizeof(real8)))
						== NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				for(i = 0; i < (*((struct_vecteur *) (*s_objet_3).objet))
						.taille; i++)
				{
					((real8 *) (*((struct_vecteur *) (*s_objet_3).objet))
							.tableau)[i] = (real8) (((integer8 *) tampon)[i]);
				}

				free((integer8 *) tampon);

				((real8 *) (*((struct_vecteur *) (*s_objet_3).objet))
						.tableau)[indice_i - 1] = (*((real8 *)
						(*s_objet_1).objet));
			}
			else if ((*s_objet_1).type == CPL)
			{
				/*
				 * Conversion du vecteur en vecteur complexe
				 */

				tampon = (void *) ((integer8 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau);

				(*((struct_vecteur *) (*s_objet_3).objet)).type = 'C';
				(*s_objet_3).type = VCX;

				if (((*((struct_vecteur *) (*s_objet_3).objet)).tableau
						= malloc(((size_t) (*((struct_vecteur *)
						(*s_objet_3).objet)).taille) *
						sizeof(struct_complexe16))) == NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				for(i = 0; i < (*((struct_vecteur *) (*s_objet_3).objet))
						.taille; i++)
				{
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_3).objet)).tableau)[i].partie_reelle =
							(real8) (((integer8 *) tampon)[i]);
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_3).objet)).tableau)[i]
							.partie_imaginaire = (real8) 0;
				}

				free((integer8 *) tampon);

				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		.partie_reelle = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_reelle;
				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		.partie_imaginaire = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_imaginaire;
			}
			else
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}
		}
		else if ((*s_objet_3).type == VRL)
		{
			/*
			 * Vecteur de rels
			 */

			if ((*s_objet_1).type == INT)
			{
				/*
				 * Conversion de l'lment  insrer en rel
				 */

				((real8 *) (*((struct_vecteur *) (*s_objet_3).objet))
						.tableau)[indice_i - 1] = (real8) (*((integer8 *)
						(*s_objet_1).objet));
			}
			else if ((*s_objet_1).type == REL)
			{
				/*
				 * Aucune conversion de type
				 */

				((real8 *) (*((struct_vecteur *) (*s_objet_3).objet))
						.tableau)[indice_i - 1] = (*((real8 *)
						(*s_objet_1).objet));
			}
			else if ((*s_objet_1).type == CPL)
			{
				/*
				 * Conversion du vecteur en vecteur complexe
				 */

				tampon = (void *) ((integer8 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau);

				(*((struct_vecteur *) (*s_objet_3).objet)).type = 'C';
				(*s_objet_3).type = VCX;

				if (((*((struct_vecteur *) (*s_objet_3).objet)).tableau
						= malloc(((size_t) (*((struct_vecteur *)
						(*s_objet_3).objet)).taille) *
						sizeof(struct_complexe16))) == NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				for(i = 0; i < (*((struct_vecteur *) (*s_objet_3).objet))
						.taille; i++)
				{
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_3).objet)).tableau)[i].partie_reelle =
							((real8 *) tampon)[i];
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_3).objet)).tableau)[i]
							.partie_imaginaire = (real8) 0;
				}

				free((real8 *) tampon);

				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		.partie_reelle = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_reelle;
				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		.partie_imaginaire = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_imaginaire;
			}
			else
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}
		}
		else
		{
			/*
			 * Vecteur de complexes
			 */

			if ((*s_objet_1).type == INT)
			{
				/*
				 * Conversion de l'lment  insrer en complexe
				 */

				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		.partie_reelle = (real8) (*((integer8 *)
						(*s_objet_1).objet));
				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1] 
						.partie_imaginaire = (real8) 0;
			}
			else if ((*s_objet_1).type == REL)
			{
				/*
				 * Conversion de l'lment  insrer en complexe
				 */

				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		.partie_reelle = (*((real8 *) (*s_objet_1).objet));
				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1] 
						.partie_imaginaire = (real8) 0;
			}
			else if ((*s_objet_1).type == CPL)
			{
				/*
				 * Aucune conversion de type
				 */

				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		.partie_reelle = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_reelle;
				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		.partie_imaginaire = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_imaginaire;
			}
			else
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}
		}

		liberation(s_etat_processus, s_objet_1);
		liberation(s_etat_processus, s_objet_2);

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_3) == d_erreur)
		{
			return;
		}
	}

/*
--------------------------------------------------------------------------------
  Traitement des lments des matrices
--------------------------------------------------------------------------------
*/

	else if (((*s_objet_3).type == MIN) ||
			((*s_objet_3).type == MRL) ||
			((*s_objet_3).type == MCX))
	{
		if ((*s_objet_2).type != LST)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
			return;
		}

		l_element_courant = (*s_objet_2).objet;
		nombre_dimensions = 0;

		while(l_element_courant != NULL)
		{
			nombre_dimensions++;
			l_element_courant = (*l_element_courant).suivant;
		}

		if (nombre_dimensions != 2)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		l_element_courant = (*s_objet_2).objet;

		indice_i = 0;
		indice_j = 0;

		while(l_element_courant != NULL)
		{
			if ((*(*l_element_courant).donnee).type != INT)
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}

			if ((*((integer8 *) (*(*l_element_courant).donnee).objet)) <= 0)
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution = d_ex_argument_invalide;
				return;
			}

			if (indice_i == 0)
			{
				indice_i = (*((integer8 *)
						(*(*l_element_courant).donnee).objet));
			}
			else
			{
				indice_j = (*((integer8 *)
						(*(*l_element_courant).donnee).objet));
			}

			l_element_courant = (*l_element_courant).suivant;
		}

		if ((indice_i > (*((struct_matrice *) (*s_objet_3).objet))
				.nombre_lignes) || (indice_j > (*((struct_matrice *)
				(*s_objet_3).objet)).nombre_colonnes))
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
			return;
		}

		if ((s_copie_3 = copie_objet(s_etat_processus, s_objet_3, 'Q')) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		liberation(s_etat_processus, s_objet_3);
		s_objet_3 = s_copie_3;

		if ((*s_objet_3).type == MIN)
		{
			/*
			 * Matrice d'entiers
			 */

			if ((*s_objet_1).type == INT)
			{
				/*
				 * Aucune conversion de type
				 */

				((integer8 **) (*((struct_matrice *) (*s_objet_3).objet))
						.tableau)[indice_i - 1][indice_j - 1] =
						(*((integer8 *) (*s_objet_1).objet));
			}
			else if ((*s_objet_1).type == REL)
			{
				/*
				 * Conversion de la matrice en matrice relle
				 */

				tampon = (void *) ((integer8 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau);

				(*((struct_matrice *) (*s_objet_3).objet)).type = 'R';
				(*s_objet_3).type = MRL;

				if (((*((struct_matrice *) (*s_objet_3).objet)).tableau
						= malloc(((size_t) (*((struct_matrice *)
						(*s_objet_3).objet)).nombre_lignes) * sizeof(real8 *)))
						== NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				for(i = 0; i < (*((struct_matrice *) (*s_objet_3).objet))
						.nombre_lignes; i++)
				{
					if ((((real8 **) (*((struct_matrice *)
							(*s_objet_3).objet)).tableau)[i]
							= malloc(((size_t) (*((struct_matrice *)
							(*s_objet_3).objet)).nombre_colonnes) *
							sizeof(real8))) == NULL)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(j = 0; j < (*((struct_matrice *) (*s_objet_3).objet))
							.nombre_colonnes; j++)
					{
						((real8 **) (*((struct_matrice *) (*s_objet_3).objet))
								.tableau)[i][j] = (real8) (((integer8 **)
								tampon)[i][j]);
					}

					free(((integer8 **) tampon)[i]);
				}

				free((integer8 **) tampon);

				((real8 **) (*((struct_matrice *) (*s_objet_3).objet))
						.tableau)[indice_i - 1][indice_j - 1] = (*((real8 *)
						(*s_objet_1).objet));
			}
			else if ((*s_objet_1).type == CPL)
			{
				/*
				 * Conversion de la matrice en matrice complexe
				 */

				tampon = (void *) ((integer8 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau);

				(*((struct_matrice *) (*s_objet_3).objet)).type = 'C';
				(*s_objet_3).type = MCX;

				if (((*((struct_matrice *) (*s_objet_3).objet)).tableau
						= malloc(((size_t) (*((struct_matrice *)
						(*s_objet_3).objet)).nombre_lignes) *
						sizeof(struct_complexe16 *))) == NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				for(i = 0; i < (*((struct_matrice *) (*s_objet_3).objet))
						.nombre_lignes; i++)
				{
					if ((((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_3).objet)).tableau)[i]
							= malloc(((size_t) (*((struct_matrice *)
							(*s_objet_3).objet)).nombre_colonnes) *
							sizeof(struct_complexe16))) == NULL)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(j = 0; j < (*((struct_matrice *) (*s_objet_3).objet))
							.nombre_colonnes; j++)
					{
						((struct_complexe16 **) (*((struct_matrice *)
								(*s_objet_3).objet)).tableau)[i][j]
						   		.partie_reelle = (real8) (((integer8 **)
								tampon)[i][j]);
						((struct_complexe16 **) (*((struct_matrice *)
								(*s_objet_3).objet)).tableau)[i][j]
						   		.partie_imaginaire = (real8) 0;
					}

					free(((integer8 **) tampon)[i]);
				}

				free((integer8 **) tampon);

				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
						[indice_j - 1].partie_reelle = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_reelle;
				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
						[indice_j - 1].partie_imaginaire =
						(*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_imaginaire;
			}
			else
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}
		}
		else if ((*s_objet_3).type == MRL)
		{
			/*
			 * Matrice de rels
			 */

			if ((*s_objet_1).type == INT)
			{
				/*
				 * Conversion de l'lment  insrer en rel
				 */

				((real8 **) (*((struct_matrice *) (*s_objet_3).objet))
						.tableau)[indice_i - 1][indice_j - 1] =
						(real8) (*((integer8 *) (*s_objet_1).objet));
			}
			else if ((*s_objet_1).type == REL)
			{
				/*
				 * Aucune conversion de type
				 */

				((real8 **) (*((struct_matrice *) (*s_objet_3).objet))
						.tableau)[indice_i - 1][indice_j - 1] = (*((real8 *)
						(*s_objet_1).objet));
			}
			else if ((*s_objet_1).type == CPL)
			{
				/*
				 * Conversion de la matrice en matrice complexe
				 */

				tampon = (void *) ((real8 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau);

				(*((struct_matrice *) (*s_objet_3).objet)).type = 'C';
				(*s_objet_3).type = MCX;

				if (((*((struct_matrice *) (*s_objet_3).objet)).tableau
						= malloc(((size_t) (*((struct_matrice *)
						(*s_objet_3).objet)).nombre_lignes) *
						sizeof(struct_complexe16 *))) == NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				for(i = 0; i < (*((struct_matrice *) (*s_objet_3).objet))
						.nombre_lignes; i++)
				{
					if ((((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_3).objet)).tableau)[i]
							= malloc(((size_t) (*((struct_matrice *)
							(*s_objet_3).objet)).nombre_colonnes) *
							sizeof(struct_complexe16))) == NULL)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(j = 0; j < (*((struct_matrice *) (*s_objet_3).objet))
							.nombre_colonnes; j++)
					{
						((struct_complexe16 **) (*((struct_matrice *)
								(*s_objet_3).objet)).tableau)[i][j]
						   		.partie_reelle = (((real8 **)
								tampon)[i][j]);
						((struct_complexe16 **) (*((struct_matrice *)
								(*s_objet_3).objet)).tableau)[i][j]
						   		.partie_imaginaire = (real8) 0;
					}

					free(((integer8 **) tampon)[i]);
				}

				free((integer8 **) tampon);

				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
						[indice_j - 1].partie_reelle = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_reelle;
				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
						[indice_j - 1].partie_imaginaire =
						(*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_imaginaire;
			}
			else
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}
		}
		else
		{
			/*
			 * Matrice de complexes
			 */

			if ((*s_objet_1).type == INT)
			{
				/*
				 * Conversion de l'lment  insrer en complexe
				 */

				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		[indice_j - 1].partie_reelle = (real8) (*((integer8 *)
						(*s_objet_1).objet));
				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1] 
						[indice_j - 1].partie_imaginaire = (real8) 0;
			}
			else if ((*s_objet_1).type == REL)
			{
				/*
				 * Conversion de l'lment  insrer en complexe
				 */

				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		[indice_j - 1].partie_reelle =
						(*((real8 *) (*s_objet_1).objet));
				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1] 
						[indice_j - 1].partie_imaginaire = (real8) 0;
			}
			else if ((*s_objet_1).type == CPL)
			{
				/*
				 * Aucune conversion de type
				 */

				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		[indice_j - 1].partie_reelle = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_reelle;
				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		[indice_j - 1].partie_imaginaire =
						(*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_imaginaire;
			}
			else
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}
		}

		liberation(s_etat_processus, s_objet_1);
		liberation(s_etat_processus, s_objet_2);

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_3) == d_erreur)
		{
			return;
		}
	}

/*
--------------------------------------------------------------------------------
  Traitement des lments des listes
--------------------------------------------------------------------------------
*/

	else if ((*s_objet_3).type == LST)
	{
		if ((*s_objet_2).type != INT)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
			return;
		}

		indice_i = (*((integer8 *) (*s_objet_2).objet));
		indice_j = 1;

		if ((*s_objet_3).nombre_occurrences > 1)
		{
			if ((s_copie_3 = copie_objet(s_etat_processus, s_objet_3, 'N'))
					== NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			liberation(s_etat_processus, s_objet_3);
			s_objet_3 = s_copie_3;
		}

		l_element_courant = (*s_objet_3).objet;

		while((l_element_courant != NULL) && (indice_j != indice_i))
		{
			l_element_courant = (*l_element_courant).suivant;
			indice_j++;
		}

		if (l_element_courant != NULL)
		{
			liberation(s_etat_processus, (*l_element_courant).donnee);
			(*l_element_courant).donnee = s_objet_1;
		}
		else
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
			return;
		}

		liberation(s_etat_processus, s_objet_2);

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_3) == d_erreur)
		{
			return;
		}
	}

/*
--------------------------------------------------------------------------------
  Traitement des lments des tables
--------------------------------------------------------------------------------
*/

	else if ((*s_objet_3).type == TBL)
	{
		if ((*s_objet_2).type != LST)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
			return;
		}

		if ((*s_objet_3).nombre_occurrences > 1)
		{
			if ((s_copie_3 = copie_objet(s_etat_processus, s_objet_3, 'N'))
					== NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			liberation(s_etat_processus, s_objet_3);
			s_objet_3 = s_copie_3;
		}

		s_objet_element = s_objet_3;
		l_element_courant = (*s_objet_2).objet;
		indice_i = 0;

		while(l_element_courant != NULL)
		{
			if ((*(*l_element_courant).donnee).type != INT)
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}

			if ((*s_objet_element).type != TBL)
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
				return;
			}

			indice_i = (*((integer8 *) (*(*l_element_courant).donnee).objet));

			if ((indice_i < 1) || (indice_i > (*((struct_tableau *)
					(*s_objet_element).objet)).nombre_elements))
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
				return;
			}

			if ((*l_element_courant).suivant != NULL)
			{
				s_objet_element = (*((struct_tableau *) (*s_objet_element)
						.objet)).elements[indice_i - 1];
			}

			l_element_courant = (*l_element_courant).suivant;
		}

		liberation(s_etat_processus, (*((struct_tableau *)
				(*s_objet_element).objet)).elements[indice_i - 1]);
		(*((struct_tableau *) (*s_objet_element).objet)).elements[indice_i - 1]
				= s_objet_1;

		liberation(s_etat_processus, s_objet_2);

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_3) == d_erreur)
		{
			return;
		}
	}

/*
--------------------------------------------------------------------------------
  Traitement des noms
--------------------------------------------------------------------------------
*/

	else if ((*s_objet_3).type == NOM)
	{
		variable_partagee = d_faux;

		if (recherche_variable(s_etat_processus, (*((struct_nom *)
				(*s_objet_3).objet)).nom) == d_faux)
		{
			(*s_etat_processus).erreur_systeme = d_es;
			(*s_etat_processus).erreur_execution = d_ex_variable_non_definie;

			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			return;
		}

		if ((*(*s_etat_processus).pointeur_variable_courante)
				.variable_verrouillee == d_vrai)
		{
			(*s_etat_processus).erreur_execution = d_ex_variable_verrouillee;

			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			return;
		}

		s_objet_4 = (*(*s_etat_processus).pointeur_variable_courante).objet;

		if (s_objet_4 == NULL)
		{
			if (recherche_variable_partagee(s_etat_processus,
					(*(*s_etat_processus).pointeur_variable_courante).nom,
					(*(*s_etat_processus).pointeur_variable_courante)
					.variable_partagee, (*(*s_etat_processus)
					.pointeur_variable_courante).origine) == NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es;
				(*s_etat_processus).erreur_execution =
						d_ex_variable_non_definie;

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				return;
			}

			s_objet_4 = (*(*s_etat_processus)
					.pointeur_variable_partagee_courante).objet;
			variable_partagee = d_vrai;
		}

		if (((*s_objet_4).type == VIN) ||
				((*s_objet_4).type == VRL) ||
				((*s_objet_4).type == VCX))
		{
			if ((*s_objet_2).type != LST)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}

			l_element_courant = (*s_objet_2).objet;
			nombre_dimensions = 0;

			while(l_element_courant != NULL)
			{
				nombre_dimensions++;
				l_element_courant = (*l_element_courant).suivant;
			}

			if (nombre_dimensions != 1)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_dimensions_invalides;
				return;
			}

			l_element_courant = (*s_objet_2).objet;

			if ((*(*l_element_courant).donnee).type != INT)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}

			if ((*((integer8 *) (*(*l_element_courant).donnee).objet)) <= 0)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution = d_ex_argument_invalide;
				return;
			}
			else if ((*((integer8 *) (*(*l_element_courant).donnee).objet)) >
					(integer8) (*((struct_vecteur *) (*s_objet_4).objet))
					.taille)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
				return;
			}

			indice_i = (*((integer8 *) (*(*l_element_courant).donnee).objet));

			if ((s_copie_4 = copie_objet(s_etat_processus, s_objet_4, 'Q'))
					== NULL)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			liberation(s_etat_processus, s_objet_4);
			s_objet_4 = s_copie_4;

			if ((*s_objet_4).type == VIN)
			{
				/*
				 * Vecteur d'entiers
				 */

				if ((*s_objet_1).type == INT)
				{
					/*
					 * Aucune conversion de type
					 */

					((integer8 *) (*((struct_vecteur *) (*s_objet_4).objet))
							.tableau)[indice_i - 1] = (*((integer8 *)
							(*s_objet_1).objet));
				}
				else if ((*s_objet_1).type == REL)
				{
					/*
					 * Conversion du vecteur en vecteur rel
					 */

					tampon = (void *) ((integer8 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau);

					(*((struct_vecteur *) (*s_objet_4).objet)).type = 'R';
					(*s_objet_4).type = VRL;

					if (((*((struct_vecteur *) (*s_objet_4).objet)).tableau
							= malloc(((size_t) (*((struct_vecteur *)
							(*s_objet_4).objet)).taille) * sizeof(real8)))
							== NULL)
					{
						if (variable_partagee == d_vrai)
						{
							if (pthread_mutex_unlock(&((*(*s_etat_processus)
									.pointeur_variable_partagee_courante)
									.mutex)) != 0)
							{
								(*s_etat_processus).erreur_systeme =
										d_es_processus;
								return;
							}
						}

						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(i = 0; i < (*((struct_vecteur *) (*s_objet_4).objet))
							.taille; i++)
					{
						((real8 *) (*((struct_vecteur *) (*s_objet_4).objet))
								.tableau)[i] = (real8) (((integer8 *)
								tampon)[i]);
					}

					free((integer8 *) tampon);

					((real8 *) (*((struct_vecteur *) (*s_objet_4).objet))
							.tableau)[indice_i - 1] = (*((real8 *)
							(*s_objet_1).objet));
				}
				else if ((*s_objet_1).type == CPL)
				{
					/*
					 * Conversion du vecteur en vecteur complexe
					 */

					tampon = (void *) ((integer8 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau);

					(*((struct_vecteur *) (*s_objet_4).objet)).type = 'C';
					(*s_objet_4).type = VCX;

					if (((*((struct_vecteur *) (*s_objet_4).objet)).tableau
							= malloc(((size_t) (*((struct_vecteur *)
							(*s_objet_4).objet)).taille) *
							sizeof(struct_complexe16))) == NULL)
					{
						if (variable_partagee == d_vrai)
						{
							if (pthread_mutex_unlock(&((*(*s_etat_processus)
									.pointeur_variable_partagee_courante)
									.mutex)) != 0)
							{
								(*s_etat_processus).erreur_systeme =
										d_es_processus;
								return;
							}
						}

						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(i = 0; i < (*((struct_vecteur *) (*s_objet_4).objet))
							.taille; i++)
					{
						((struct_complexe16 *) (*((struct_vecteur *)
								(*s_objet_4).objet)).tableau)[i].partie_reelle =
								(real8) (((integer8 *) tampon)[i]);
						((struct_complexe16 *) (*((struct_vecteur *)
								(*s_objet_4).objet)).tableau)[i]
								.partie_imaginaire = (real8) 0;
					}

					free((integer8 *) tampon);

					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							.partie_reelle = (*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_reelle;
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							.partie_imaginaire = (*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_imaginaire;
				}
				else
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}
			}
			else if ((*s_objet_4).type == VRL)
			{
				/*
				 * Vecteur de rels
				 */

				if ((*s_objet_1).type == INT)
				{
					/*
					 * Conversion de l'lment  insrer en rel
					 */

					((real8 *) (*((struct_vecteur *) (*s_objet_4).objet))
							.tableau)[indice_i - 1] = (real8) (*((integer8 *)
							(*s_objet_1).objet));
				}
				else if ((*s_objet_1).type == REL)
				{
					/*
					 * Aucune conversion de type
					 */

					((real8 *) (*((struct_vecteur *) (*s_objet_4).objet))
							.tableau)[indice_i - 1] = (*((real8 *)
							(*s_objet_1).objet));
				}
				else if ((*s_objet_1).type == CPL)
				{
					/*
					 * Conversion du vecteur en vecteur complexe
					 */

					tampon = (void *) ((integer8 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau);

					(*((struct_vecteur *) (*s_objet_4).objet)).type = 'C';
					(*s_objet_4).type = VCX;

					if (((*((struct_vecteur *) (*s_objet_4).objet)).tableau
							= malloc(((size_t) (*((struct_vecteur *)
							(*s_objet_4).objet)).taille) *
							sizeof(struct_complexe16))) == NULL)
					{
						if (variable_partagee == d_vrai)
						{
							if (pthread_mutex_unlock(&((*(*s_etat_processus)
									.pointeur_variable_partagee_courante)
									.mutex)) != 0)
							{
								(*s_etat_processus).erreur_systeme =
										d_es_processus;
								return;
							}
						}

						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(i = 0; i < (*((struct_vecteur *) (*s_objet_4).objet))
							.taille; i++)
					{
						((struct_complexe16 *) (*((struct_vecteur *)
								(*s_objet_4).objet)).tableau)[i].partie_reelle =
								((real8 *) tampon)[i];
						((struct_complexe16 *) (*((struct_vecteur *)
								(*s_objet_4).objet)).tableau)[i]
								.partie_imaginaire = (real8) 0;
					}

					free((real8 *) tampon);

					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							.partie_reelle = (*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_reelle;
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							.partie_imaginaire = (*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_imaginaire;
				}
				else
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}
			}
			else
			{
				/*
				 * Vecteur de complexes
				 */

				if ((*s_objet_1).type == INT)
				{
					/*
					 * Conversion de l'lment  insrer en complexe
					 */

					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							.partie_reelle = (real8) (*((integer8 *)
							(*s_objet_1).objet));
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1] 
							.partie_imaginaire = (real8) 0;
				}
				else if ((*s_objet_1).type == REL)
				{
					/*
					 * Conversion de l'lment  insrer en complexe
					 */

					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							.partie_reelle = (*((real8 *) (*s_objet_1).objet));
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1] 
							.partie_imaginaire = (real8) 0;
				}
				else if ((*s_objet_1).type == CPL)
				{
					/*
					 * Aucune conversion de type
					 */

					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							.partie_reelle = (*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_reelle;
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							.partie_imaginaire = (*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_imaginaire;
				}
				else
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}
			}

			if (variable_partagee == d_faux)
			{
				(*(*s_etat_processus).pointeur_variable_courante).objet =
						s_objet_4;
			}
			else
			{
				(*(*s_etat_processus).pointeur_variable_courante).objet = NULL;
				(*(*s_etat_processus).pointeur_variable_partagee_courante)
						.objet = s_objet_4;

				if (pthread_mutex_unlock(&((*(*s_etat_processus)
						.pointeur_variable_partagee_courante).mutex)) != 0)
				{
					(*s_etat_processus).erreur_systeme = d_es_processus;
					return;
				}
			}

			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);
		}
		else if (((*s_objet_4).type == MIN) ||
				((*s_objet_4).type == MRL) ||
				((*s_objet_4).type == MCX))
		{
			if ((*s_objet_2).type != LST)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}

			l_element_courant = (*s_objet_2).objet;
			nombre_dimensions = 0;

			while(l_element_courant != NULL)
			{
				nombre_dimensions++;
				l_element_courant = (*l_element_courant).suivant;
			}

			if (nombre_dimensions != 2)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_dimensions_invalides;
				return;
			}

			l_element_courant = (*s_objet_2).objet;

			indice_i = 0;
			indice_j = 0;

			while(l_element_courant != NULL)
			{
				if ((*(*l_element_courant).donnee).type != INT)
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme = d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}

				if ((*((integer8 *) (*(*l_element_courant).donnee).objet))
						<= 0)
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme = d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_argument_invalide;
					return;
				}

				if (indice_i == 0)
				{
					indice_i = (*((integer8 *)
							(*(*l_element_courant).donnee).objet));
				}
				else
				{
					indice_j = (*((integer8 *)
							(*(*l_element_courant).donnee).objet));
				}

				l_element_courant = (*l_element_courant).suivant;
			}

			if ((indice_i > (*((struct_matrice *) (*s_objet_4).objet))
					.nombre_lignes) || (indice_j > (*((struct_matrice *)
					(*s_objet_4).objet)).nombre_colonnes))
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
				return;
			}

			if ((s_copie_4 = copie_objet(s_etat_processus, s_objet_4, 'Q'))
					== NULL)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			liberation(s_etat_processus, s_objet_4);
			s_objet_4 = s_copie_4;

			if ((*s_objet_4).type == MIN)
			{
				/*
				 * Matrice d'entiers
				 */

				if ((*s_objet_1).type == INT)
				{
					/*
					 * Aucune conversion de type
					 */

					((integer8 **) (*((struct_matrice *) (*s_objet_4).objet))
							.tableau)[indice_i - 1][indice_j - 1] =
							(*((integer8 *) (*s_objet_1).objet));
				}
				else if ((*s_objet_1).type == REL)
				{
					/*
					 * Conversion de la matrice en matrice relle
					 */

					tampon = (void *) ((integer8 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau);

					(*((struct_matrice *) (*s_objet_4).objet)).type = 'R';
					(*s_objet_4).type = MRL;

					if (((*((struct_matrice *) (*s_objet_4).objet)).tableau
							= malloc(((size_t) (*((struct_matrice *)
							(*s_objet_4).objet)).nombre_lignes) *
							sizeof(real8 *))) == NULL)
					{
						if (variable_partagee == d_vrai)
						{
							if (pthread_mutex_unlock(&((*(*s_etat_processus)
									.pointeur_variable_partagee_courante)
									.mutex)) != 0)
							{
								(*s_etat_processus).erreur_systeme =
										d_es_processus;
								return;
							}
						}

						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(i = 0; i < (*((struct_matrice *) (*s_objet_4).objet))
							.nombre_lignes; i++)
					{
						if ((((real8 **) (*((struct_matrice *)
								(*s_objet_4).objet)).tableau)[i]
								= malloc(((size_t) (*((struct_matrice *)
								(*s_objet_4).objet)).nombre_colonnes) *
								sizeof(real8))) == NULL)
						{
							if (variable_partagee == d_vrai)
							{
								if (pthread_mutex_unlock(&((*(*s_etat_processus)
										.pointeur_variable_partagee_courante)
										.mutex)) != 0)
								{
									(*s_etat_processus).erreur_systeme =
											d_es_processus;
									return;
								}
							}

							(*s_etat_processus).erreur_systeme =
									d_es_allocation_memoire;
							return;
						}

						for(j = 0; j < (*((struct_matrice *)
								(*s_objet_4).objet)).nombre_colonnes; j++)
						{
							((real8 **) (*((struct_matrice *) (*s_objet_4)
									.objet)).tableau)[i][j] = (real8)
									(((integer8 **) tampon)[i][j]);
						}

						free(((integer8 **) tampon)[i]);
					}

					free((integer8 **) tampon);

					((real8 **) (*((struct_matrice *) (*s_objet_4).objet))
							.tableau)[indice_i - 1][indice_j - 1] =
							(*((real8 *) (*s_objet_1).objet));
				}
				else if ((*s_objet_1).type == CPL)
				{
					/*
					 * Conversion de la matrice en matrice complexe
					 */

					tampon = (void *) ((integer8 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau);

					(*((struct_matrice *) (*s_objet_4).objet)).type = 'C';
					(*s_objet_4).type = MCX;

					if (((*((struct_matrice *) (*s_objet_4).objet)).tableau
							= malloc(((size_t) (*((struct_matrice *)
							(*s_objet_4).objet)).nombre_lignes) *
							sizeof(struct_complexe16 *))) == NULL)
					{
						if (variable_partagee == d_vrai)
						{
							if (pthread_mutex_unlock(&((*(*s_etat_processus)
									.pointeur_variable_partagee_courante)
									.mutex)) != 0)
							{
								(*s_etat_processus).erreur_systeme =
										d_es_processus;
								return;
							}
						}

						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(i = 0; i < (*((struct_matrice *) (*s_objet_4).objet))
							.nombre_lignes; i++)
					{
						if ((((struct_complexe16 **) (*((struct_matrice *)
								(*s_objet_4).objet)).tableau)[i]
								= malloc(((size_t) (*((struct_matrice *)
								(*s_objet_4).objet)).nombre_colonnes) *
								sizeof(struct_complexe16))) == NULL)
						{
							if (variable_partagee == d_vrai)
							{
								if (pthread_mutex_unlock(&((*(*s_etat_processus)
										.pointeur_variable_partagee_courante)
										.mutex)) != 0)
								{
									(*s_etat_processus).erreur_systeme =
											d_es_processus;
									return;
								}
							}

							(*s_etat_processus).erreur_systeme =
									d_es_allocation_memoire;
							return;
						}

						for(j = 0; j < (*((struct_matrice *)
								(*s_objet_4).objet)).nombre_colonnes; j++)
						{
							((struct_complexe16 **) (*((struct_matrice *)
									(*s_objet_4).objet)).tableau)[i][j]
									.partie_reelle = (real8) (((integer8 **)
									tampon)[i][j]);
							((struct_complexe16 **) (*((struct_matrice *)
									(*s_objet_4).objet)).tableau)[i][j]
									.partie_imaginaire = (real8) 0;
						}

						free(((integer8 **) tampon)[i]);
					}

					free((integer8 **) tampon);

					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							[indice_j - 1].partie_reelle =
							(*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_reelle;
					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							[indice_j - 1].partie_imaginaire =
							(*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_imaginaire;
				}
				else
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}
			}
			else if ((*s_objet_4).type == MRL)
			{
				/*
				 * Matrice de rels
				 */

				if ((*s_objet_1).type == INT)
				{
					/*
					 * Conversion de l'lment  insrer en rel
					 */

					((real8 **) (*((struct_matrice *) (*s_objet_4).objet))
							.tableau)[indice_i - 1][indice_j - 1] =
							(real8) (*((integer8 *) (*s_objet_1).objet));
				}
				else if ((*s_objet_1).type == REL)
				{
					/*
					 * Aucune conversion de type
					 */

					((real8 **) (*((struct_matrice *) (*s_objet_4).objet))
							.tableau)[indice_i - 1][indice_j - 1] =
							(*((real8 *) (*s_objet_1).objet));
				}
				else if ((*s_objet_1).type == CPL)
				{
					/*
					 * Conversion de la matrice en matrice complexe
					 */

					tampon = (void *) ((real8 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau);

					(*((struct_matrice *) (*s_objet_4).objet)).type = 'C';
					(*s_objet_4).type = MCX;

					if (((*((struct_matrice *) (*s_objet_4).objet)).tableau
							= malloc(((size_t) (*((struct_matrice *)
							(*s_objet_4).objet)).nombre_lignes) *
							sizeof(struct_complexe16 *))) == NULL)
					{
						if (variable_partagee == d_vrai)
						{
							if (pthread_mutex_unlock(&((*(*s_etat_processus)
									.pointeur_variable_partagee_courante)
									.mutex)) != 0)
							{
								(*s_etat_processus).erreur_systeme =
										d_es_processus;
								return;
							}
						}

						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(i = 0; i < (*((struct_matrice *) (*s_objet_4).objet))
							.nombre_lignes; i++)
					{
						if ((((struct_complexe16 **) (*((struct_matrice *)
								(*s_objet_4).objet)).tableau)[i]
								= malloc(((size_t) (*((struct_matrice *)
								(*s_objet_4).objet)).nombre_colonnes) *
								sizeof(struct_complexe16))) == NULL)
						{
							if (variable_partagee == d_vrai)
							{
								if (pthread_mutex_unlock(&((*(*s_etat_processus)
										.pointeur_variable_partagee_courante)
										.mutex)) != 0)
								{
									(*s_etat_processus).erreur_systeme =
											d_es_processus;
									return;
								}
							}

							(*s_etat_processus).erreur_systeme =
									d_es_allocation_memoire;
							return;
						}

						for(j = 0; j < (*((struct_matrice *)
								(*s_objet_4).objet)).nombre_colonnes; j++)
						{
							((struct_complexe16 **) (*((struct_matrice *)
									(*s_objet_4).objet)).tableau)[i][j]
									.partie_reelle = (((real8 **)
									tampon)[i][j]);
							((struct_complexe16 **) (*((struct_matrice *)
									(*s_objet_4).objet)).tableau)[i][j]
									.partie_imaginaire = (real8) 0;
						}

						free(((integer8 **) tampon)[i]);
					}

					free((integer8 **) tampon);

					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							[indice_j - 1].partie_reelle =
							(*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_reelle;
					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							[indice_j - 1].partie_imaginaire =
							(*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_imaginaire;
				}
				else
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}
			}
			else
			{
				/*
				 * Matrice de complexes
				 */

				if ((*s_objet_1).type == INT)
				{
					/*
					 * Conversion de l'lment  insrer en complexe
					 */

					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							[indice_j - 1].partie_reelle = (real8)
							(*((integer8 *) (*s_objet_1).objet));
					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1] 
							[indice_j - 1].partie_imaginaire = (real8) 0;
				}
				else if ((*s_objet_1).type == REL)
				{
					/*
					 * Conversion de l'lment  insrer en complexe
					 */

					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							[indice_j - 1].partie_reelle =
							(*((real8 *) (*s_objet_1).objet));
					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1] 
							[indice_j - 1].partie_imaginaire = (real8) 0;
				}
				else if ((*s_objet_1).type == CPL)
				{
					/*
					 * Aucune conversion de type
					 */

					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							[indice_j - 1].partie_reelle =
							(*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_reelle;
					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							[indice_j - 1].partie_imaginaire =
							(*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_imaginaire;
				}
				else
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}
			}

			if (variable_partagee == d_faux)
			{
				(*(*s_etat_processus).pointeur_variable_courante).objet =
						s_objet_4;
			}
			else
			{
				(*(*s_etat_processus).pointeur_variable_courante).objet = NULL;
				(*(*s_etat_processus).pointeur_variable_partagee_courante)
						.objet = s_objet_4;

				if (pthread_mutex_unlock(&((*(*s_etat_processus)
						.pointeur_variable_partagee_courante).mutex)) != 0)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_processus;
					return;
				}
			}

			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);
		}
		else if ((*s_objet_4).type == LST)
		{
			if ((*s_objet_2).type != INT)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}

			indice_i = (*((integer8 *) (*s_objet_2).objet));
			indice_j = 1;

			if ((*s_objet_4).nombre_occurrences > 1)
			{
				if ((s_copie_4 = copie_objet(s_etat_processus, s_objet_4, 'N'))
						== NULL)
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				liberation(s_etat_processus, s_objet_4);
				s_objet_4 = s_copie_4;
			}

			l_element_courant = (*s_objet_4).objet;

			while((l_element_courant != NULL) && (indice_j != indice_i))
			{
				l_element_courant = (*l_element_courant).suivant;
				indice_j++;
			}

			if (l_element_courant != NULL)
			{
				liberation(s_etat_processus, (*l_element_courant).donnee);
				(*l_element_courant).donnee = s_objet_1;
			}
			else
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
				return;
			}

			if (variable_partagee == d_faux)
			{
				(*(*s_etat_processus).pointeur_variable_courante).objet =
						s_objet_4;
			}
			else
			{
				(*(*s_etat_processus).pointeur_variable_courante).objet = NULL;
				(*(*s_etat_processus).pointeur_variable_partagee_courante)
						.objet = s_objet_4;

				if (pthread_mutex_unlock(&((*(*s_etat_processus)
						.pointeur_variable_partagee_courante).mutex)) != 0)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_processus;
					return;
				}
			}

			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);
		}
		else if ((*s_objet_4).type == TBL)
		{
			if ((*s_objet_2).type != LST)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}

			if ((*s_objet_4).nombre_occurrences > 1)
			{
				if ((s_copie_4 = copie_objet(s_etat_processus, s_objet_4, 'N'))
						== NULL)
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				liberation(s_etat_processus, s_objet_4);
				s_objet_4 = s_copie_4;
			}

			s_objet_element = s_objet_4;
			l_element_courant = (*s_objet_2).objet;
			indice_i = 0;

			while(l_element_courant != NULL)
			{
				if ((*(*l_element_courant).donnee).type != INT)
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}

				if ((*s_objet_element).type != TBL)
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_element_inexistant;
					return;
				}

				indice_i = (*((integer8 *) (*(*l_element_courant)
						.donnee).objet));

				if ((indice_i < 1) || (indice_i > (*((struct_tableau *)
						(*s_objet_element).objet)).nombre_elements))
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_element_inexistant;
					return;
				}

				if ((*l_element_courant).suivant != NULL)
				{
					s_objet_element = (*((struct_tableau *) (*s_objet_element)
							.objet)).elements[indice_i - 1];
				}

				l_element_courant = (*l_element_courant).suivant;
			}

			liberation(s_etat_processus, (*((struct_tableau *)
					(*s_objet_element).objet)).elements[indice_i - 1]);
			(*((struct_tableau *) (*s_objet_element).objet)).elements
					[indice_i - 1] = s_objet_1;

			if (variable_partagee == d_faux)
			{
				(*(*s_etat_processus).pointeur_variable_courante).objet =
						s_objet_4;
			}
			else
			{
				(*(*s_etat_processus).pointeur_variable_courante).objet = NULL;
				(*(*s_etat_processus).pointeur_variable_partagee_courante)
						.objet = s_objet_4;

				if (pthread_mutex_unlock(&((*(*s_etat_processus)
						.pointeur_variable_partagee_courante).mutex)) != 0)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_processus;
					return;
				}
			}

			liberation(s_etat_processus, s_objet_2);
		}
		else
		{
			if (variable_partagee == d_vrai)
			{
				if (pthread_mutex_unlock(&((*(*s_etat_processus)
						.pointeur_variable_partagee_courante).mutex)) != 0)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_processus;
					return;
				}
			}

			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
			return;
		}
	}

/*
--------------------------------------------------------------------------------
  Arguments incompatibles
--------------------------------------------------------------------------------
*/

	else
	{
		liberation(s_etat_processus, s_objet_1);
		liberation(s_etat_processus, s_objet_2);
		liberation(s_etat_processus, s_objet_3);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	return;
}


/*
================================================================================
  Fonction 'puti'
================================================================================
  Entres : pointeur sur une structure struct_processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_puti(struct_processus *s_etat_processus)
{
	logical1							variable_partagee;

	struct_liste_chainee				*l_element_courant;

	struct_objet						*s_copie_2;
	struct_objet						*s_copie_3;
	struct_objet						*s_copie_4;
	struct_objet						*s_objet_1;
	struct_objet						*s_objet_2;
	struct_objet						*s_objet_3;
	struct_objet						*s_objet_4;

	integer8							i;
	integer8							indice_i;
	integer8							indice_j;
	integer8							j;
	integer8							nombre_dimensions;
	integer8							nombre_elements;

	void								*tampon;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  PUTI ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(change un lment)\n\n");
		}
		else
		{
			printf("(change element)\n\n");
		}

		printf("    3: %s, %s, %s, %s\n", d_VIN, d_VRL, d_VCX, d_NOM);
		printf("    2: %s\n", d_LST);
		printf("    1: %s, %s, %s\n", d_INT, d_REL, d_CPL);
		printf("->  2: %s, %s, %s, %s\n", d_VIN, d_VRL, d_VCX, d_NOM);
		printf("    1: %s\n\n", d_LST);

		printf("    3: %s, %s, %s, %s\n", d_MIN, d_MRL, d_MCX, d_NOM);
		printf("    2: %s\n", d_LST);
		printf("    1: %s, %s, %s\n", d_INT, d_REL, d_CPL);
		printf("->  2: %s, %s, %s, %s\n", d_MIN, d_MRL, d_MCX, d_NOM);
		printf("    1: %s\n\n", d_LST);

		printf("    3: %s, %s\n", d_LST, d_NOM);
		printf("    2: %s\n", d_INT);
		printf("    1: %s, %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s, %s,\n"
				"       %s, %s, %s, %s,\n"
				"       %s, %s\n",
				d_INT, d_REL, d_CPL, d_VIN, d_VRL, d_VCX, d_MIN, d_MRL, d_MCX,
				d_TAB, d_BIN, d_NOM, d_CHN, d_LST, d_ALG, d_RPN, d_FCH, d_SCK,
				d_SQL, d_SLB, d_PRC, d_MTX);
		printf("->  2: %s, %s\n", d_LST, d_NOM);
		printf("    1: %s\n", d_INT);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 3) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_1) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_2) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_1);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_3) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_1);
		liberation(s_etat_processus, s_objet_2);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if ((s_copie_2 = copie_objet(s_etat_processus, s_objet_2, 'O')) == NULL)
	{
		(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
		return;
	}

	liberation(s_etat_processus, s_objet_2);
	s_objet_2 = s_copie_2;

/*
--------------------------------------------------------------------------------
  Traitement des lments des vecteurs
--------------------------------------------------------------------------------
*/

	if (((*s_objet_3).type == VIN) ||
			((*s_objet_3).type == VRL) ||
			((*s_objet_3).type == VCX))
	{
		if ((*s_objet_2).type != LST)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
			return;
		}

		l_element_courant = (*s_objet_2).objet;
		nombre_dimensions = 0;

		while(l_element_courant != NULL)
		{
			nombre_dimensions++;
			l_element_courant = (*l_element_courant).suivant;
		}

		if (nombre_dimensions != 1)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		l_element_courant = (*s_objet_2).objet;

		if ((*(*l_element_courant).donnee).type != INT)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
			return;
		}

		if ((*((integer8 *) (*(*l_element_courant).donnee).objet)) <= 0)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_argument_invalide;
			return;
		}
		else if ((*((integer8 *) (*(*l_element_courant).donnee).objet)) >
				(integer8) (*((struct_vecteur *) (*s_objet_3).objet)).taille)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
			return;
		}

		indice_i = (*((integer8 *) (*(*l_element_courant).donnee).objet));

		if ((s_copie_3 = copie_objet(s_etat_processus, s_objet_3, 'Q')) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		liberation(s_etat_processus, s_objet_3);
		s_objet_3 = s_copie_3;

		if ((*s_objet_3).type == VIN)
		{
			/*
			 * Vecteur d'entiers
			 */

			if ((*s_objet_1).type == INT)
			{
				/*
				 * Aucune conversion de type
				 */

				((integer8 *) (*((struct_vecteur *) (*s_objet_3).objet))
						.tableau)[indice_i - 1] = (*((integer8 *)
						(*s_objet_1).objet));
			}
			else if ((*s_objet_1).type == REL)
			{
				/*
				 * Conversion du vecteur en vecteur rel
				 */

				tampon = (void *) ((integer8 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau);

				(*((struct_vecteur *) (*s_objet_3).objet)).type = 'R';
				(*s_objet_3).type = VRL;

				if (((*((struct_vecteur *) (*s_objet_3).objet)).tableau
						= malloc(((size_t) (*((struct_vecteur *)
						(*s_objet_3).objet)).taille) * sizeof(real8)))
						== NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				for(i = 0; i < (*((struct_vecteur *) (*s_objet_3).objet))
						.taille; i++)
				{
					((real8 *) (*((struct_vecteur *) (*s_objet_3).objet))
							.tableau)[i] = (real8) (((integer8 *) tampon)[i]);
				}

				free((integer8 *) tampon);

				((real8 *) (*((struct_vecteur *) (*s_objet_3).objet))
						.tableau)[indice_i - 1] = (*((real8 *)
						(*s_objet_1).objet));
			}
			else if ((*s_objet_1).type == CPL)
			{
				/*
				 * Conversion du vecteur en vecteur complexe
				 */

				tampon = (void *) ((integer8 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau);

				(*((struct_vecteur *) (*s_objet_3).objet)).type = 'C';
				(*s_objet_3).type = VCX;

				if (((*((struct_vecteur *) (*s_objet_3).objet)).tableau
						= malloc(((size_t) (*((struct_vecteur *)
						(*s_objet_3).objet)).taille) *
						sizeof(struct_complexe16))) == NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				for(i = 0; i < (*((struct_vecteur *) (*s_objet_3).objet))
						.taille; i++)
				{
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_3).objet)).tableau)[i].partie_reelle =
							(real8) (((integer8 *) tampon)[i]);
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_3).objet)).tableau)[i]
							.partie_imaginaire = (real8) 0;
				}

				free((integer8 *) tampon);

				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		.partie_reelle = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_reelle;
				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		.partie_imaginaire = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_imaginaire;
			}
			else
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}
		}
		else if ((*s_objet_3).type == VRL)
		{
			/*
			 * Vecteur de rels
			 */

			if ((*s_objet_1).type == INT)
			{
				/*
				 * Conversion de l'lment  insrer en rel
				 */

				((real8 *) (*((struct_vecteur *) (*s_objet_3).objet))
						.tableau)[indice_i - 1] = (real8) (*((integer8 *)
						(*s_objet_1).objet));
			}
			else if ((*s_objet_1).type == REL)
			{
				/*
				 * Aucune conversion de type
				 */

				((real8 *) (*((struct_vecteur *) (*s_objet_3).objet))
						.tableau)[indice_i - 1] = (*((real8 *)
						(*s_objet_1).objet));
			}
			else if ((*s_objet_1).type == CPL)
			{
				/*
				 * Conversion du vecteur en vecteur complexe
				 */

				tampon = (void *) ((integer8 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau);

				(*((struct_vecteur *) (*s_objet_3).objet)).type = 'C';
				(*s_objet_3).type = VCX;

				if (((*((struct_vecteur *) (*s_objet_3).objet)).tableau
						= malloc(((size_t) (*((struct_vecteur *)
						(*s_objet_3).objet)).taille) *
						sizeof(struct_complexe16))) == NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				for(i = 0; i < (*((struct_vecteur *) (*s_objet_3).objet))
						.taille; i++)
				{
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_3).objet)).tableau)[i].partie_reelle =
							((real8 *) tampon)[i];
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_3).objet)).tableau)[i]
							.partie_imaginaire = (real8) 0;
				}

				free((real8 *) tampon);

				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		.partie_reelle = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_reelle;
				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		.partie_imaginaire = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_imaginaire;
			}
			else
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}
		}
		else
		{
			/*
			 * Vecteur de complexes
			 */

			if ((*s_objet_1).type == INT)
			{
				/*
				 * Conversion de l'lment  insrer en complexe
				 */

				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		.partie_reelle = (real8) (*((integer8 *)
						(*s_objet_1).objet));
				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1] 
						.partie_imaginaire = (real8) 0;
			}
			else if ((*s_objet_1).type == REL)
			{
				/*
				 * Conversion de l'lment  insrer en complexe
				 */

				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		.partie_reelle = (*((real8 *) (*s_objet_1).objet));
				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1] 
						.partie_imaginaire = (real8) 0;
			}
			else if ((*s_objet_1).type == CPL)
			{
				/*
				 * Aucune conversion de type
				 */

				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		.partie_reelle = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_reelle;
				((struct_complexe16 *) (*((struct_vecteur *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		.partie_imaginaire = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_imaginaire;
			}
			else
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}
		}

		liberation(s_etat_processus, s_objet_1);

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_3) == d_erreur)
		{
			return;
		}

		(*((integer8 *) (*(*l_element_courant).donnee).objet)) =
				(indice_i % (*((struct_vecteur *) (*s_objet_3).objet))
				.taille) + 1;

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_2) == d_erreur)
		{
			return;
		}
	}

/*
--------------------------------------------------------------------------------
  Traitement des lments des matrices
--------------------------------------------------------------------------------
*/

	else if (((*s_objet_3).type == MIN) ||
			((*s_objet_3).type == MRL) ||
			((*s_objet_3).type == MCX))
	{
		if ((*s_objet_2).type != LST)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
			return;
		}

		l_element_courant = (*s_objet_2).objet;
		nombre_dimensions = 0;

		while(l_element_courant != NULL)
		{
			nombre_dimensions++;
			l_element_courant = (*l_element_courant).suivant;
		}

		if (nombre_dimensions != 2)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_dimensions_invalides;
			return;
		}

		l_element_courant = (*s_objet_2).objet;

		indice_i = 0;
		indice_j = 0;

		while(l_element_courant != NULL)
		{
			if ((*(*l_element_courant).donnee).type != INT)
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}

			if ((*((integer8 *) (*(*l_element_courant).donnee).objet)) <= 0)
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution = d_ex_argument_invalide;
				return;
			}

			if (indice_i == 0)
			{
				indice_i = (*((integer8 *)
						(*(*l_element_courant).donnee).objet));
			}
			else
			{
				indice_j = (*((integer8 *)
						(*(*l_element_courant).donnee).objet));
			}

			l_element_courant = (*l_element_courant).suivant;
		}

		if ((indice_i > (*((struct_matrice *) (*s_objet_3).objet))
				.nombre_lignes) || (indice_j > (*((struct_matrice *)
				(*s_objet_3).objet)).nombre_colonnes))
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
			return;
		}

		if ((s_copie_3 = copie_objet(s_etat_processus, s_objet_3, 'Q')) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		liberation(s_etat_processus, s_objet_3);
		s_objet_3 = s_copie_3;

		if ((*s_objet_3).type == MIN)
		{
			/*
			 * Matrice d'entiers
			 */

			if ((*s_objet_1).type == INT)
			{
				/*
				 * Aucune conversion de type
				 */

				((integer8 **) (*((struct_matrice *) (*s_objet_3).objet))
						.tableau)[indice_i - 1][indice_j - 1] =
						(*((integer8 *) (*s_objet_1).objet));
			}
			else if ((*s_objet_1).type == REL)
			{
				/*
				 * Conversion de la matrice en matrice relle
				 */

				tampon = (void *) ((integer8 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau);

				(*((struct_matrice *) (*s_objet_3).objet)).type = 'R';
				(*s_objet_3).type = MRL;

				if (((*((struct_matrice *) (*s_objet_3).objet)).tableau
						= malloc(((size_t) (*((struct_matrice *)
						(*s_objet_3).objet)).nombre_lignes) * sizeof(real8 *)))
						== NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				for(i = 0; i < (*((struct_matrice *) (*s_objet_3).objet))
						.nombre_lignes; i++)
				{
					if ((((real8 **) (*((struct_matrice *)
							(*s_objet_3).objet)).tableau)[i]
							= malloc(((size_t) (*((struct_matrice *)
							(*s_objet_3).objet)).nombre_colonnes) *
							sizeof(real8))) == NULL)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(j = 0; j < (*((struct_matrice *) (*s_objet_3).objet))
							.nombre_colonnes; j++)
					{
						((real8 **) (*((struct_matrice *) (*s_objet_3).objet))
								.tableau)[i][j] = (real8) (((integer8 **)
								tampon)[i][j]);
					}

					free(((integer8 **) tampon)[i]);
				}

				free((integer8 **) tampon);

				((real8 **) (*((struct_matrice *) (*s_objet_3).objet))
						.tableau)[indice_i - 1][indice_j - 1] = (*((real8 *)
						(*s_objet_1).objet));
			}
			else if ((*s_objet_1).type == CPL)
			{
				/*
				 * Conversion de la matrice en matrice complexe
				 */

				tampon = (void *) ((integer8 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau);

				(*((struct_matrice *) (*s_objet_3).objet)).type = 'C';
				(*s_objet_3).type = MCX;

				if (((*((struct_matrice *) (*s_objet_3).objet)).tableau
						= malloc(((size_t) (*((struct_matrice *)
						(*s_objet_3).objet)).nombre_lignes) *
						sizeof(struct_complexe16 *))) == NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				for(i = 0; i < (*((struct_matrice *) (*s_objet_3).objet))
						.nombre_lignes; i++)
				{
					if ((((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_3).objet)).tableau)[i]
							= malloc(((size_t) (*((struct_matrice *)
							(*s_objet_3).objet)).nombre_colonnes) *
							sizeof(struct_complexe16))) == NULL)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(j = 0; j < (*((struct_matrice *) (*s_objet_3).objet))
							.nombre_colonnes; j++)
					{
						((struct_complexe16 **) (*((struct_matrice *)
								(*s_objet_3).objet)).tableau)[i][j]
						   		.partie_reelle = (real8) (((integer8 **)
								tampon)[i][j]);
						((struct_complexe16 **) (*((struct_matrice *)
								(*s_objet_3).objet)).tableau)[i][j]
						   		.partie_imaginaire = (real8) 0;
					}

					free(((integer8 **) tampon)[i]);
				}

				free((integer8 **) tampon);

				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
						[indice_j - 1].partie_reelle = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_reelle;
				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
						[indice_j - 1].partie_imaginaire =
						(*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_imaginaire;
			}
			else
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}
		}
		else if ((*s_objet_3).type == MRL)
		{
			/*
			 * Matrice de rels
			 */

			if ((*s_objet_1).type == INT)
			{
				/*
				 * Conversion de l'lment  insrer en rel
				 */

				((real8 **) (*((struct_matrice *) (*s_objet_3).objet))
						.tableau)[indice_i - 1][indice_j - 1] =
						(real8) (*((integer8 *) (*s_objet_1).objet));
			}
			else if ((*s_objet_1).type == REL)
			{
				/*
				 * Aucune conversion de type
				 */

				((real8 **) (*((struct_matrice *) (*s_objet_3).objet))
						.tableau)[indice_i - 1][indice_j - 1] = (*((real8 *)
						(*s_objet_1).objet));
			}
			else if ((*s_objet_1).type == CPL)
			{
				/*
				 * Conversion de la matrice en matrice complexe
				 */

				tampon = (void *) ((real8 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau);

				(*((struct_matrice *) (*s_objet_3).objet)).type = 'C';
				(*s_objet_3).type = MCX;

				if (((*((struct_matrice *) (*s_objet_3).objet)).tableau
						= malloc(((size_t) (*((struct_matrice *)
						(*s_objet_3).objet)).nombre_lignes) *
						sizeof(struct_complexe16 *))) == NULL)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				for(i = 0; i < (*((struct_matrice *) (*s_objet_3).objet))
						.nombre_lignes; i++)
				{
					if ((((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_3).objet)).tableau)[i]
							= malloc(((size_t) (*((struct_matrice *)
							(*s_objet_3).objet)).nombre_colonnes) *
							sizeof(struct_complexe16))) == NULL)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(j = 0; j < (*((struct_matrice *) (*s_objet_3).objet))
							.nombre_colonnes; j++)
					{
						((struct_complexe16 **) (*((struct_matrice *)
								(*s_objet_3).objet)).tableau)[i][j]
						   		.partie_reelle = (((real8 **)
								tampon)[i][j]);
						((struct_complexe16 **) (*((struct_matrice *)
								(*s_objet_3).objet)).tableau)[i][j]
						   		.partie_imaginaire = (real8) 0;
					}

					free(((integer8 **) tampon)[i]);
				}

				free((integer8 **) tampon);

				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
						[indice_j - 1].partie_reelle = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_reelle;
				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
						[indice_j - 1].partie_imaginaire =
						(*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_imaginaire;
			}
			else
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}
		}
		else
		{
			/*
			 * Matrice de complexes
			 */

			if ((*s_objet_1).type == INT)
			{
				/*
				 * Conversion de l'lment  insrer en complexe
				 */

				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		[indice_j - 1].partie_reelle = (real8) (*((integer8 *)
						(*s_objet_1).objet));
				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1] 
						[indice_j - 1].partie_imaginaire = (real8) 0;
			}
			else if ((*s_objet_1).type == REL)
			{
				/*
				 * Conversion de l'lment  insrer en complexe
				 */

				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		[indice_j - 1].partie_reelle =
						(*((real8 *) (*s_objet_1).objet));
				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1] 
						[indice_j - 1].partie_imaginaire = (real8) 0;
			}
			else if ((*s_objet_1).type == CPL)
			{
				/*
				 * Aucune conversion de type
				 */

				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		[indice_j - 1].partie_reelle = (*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_reelle;
				((struct_complexe16 **) (*((struct_matrice *)
						(*s_objet_3).objet)).tableau)[indice_i - 1]
				   		[indice_j - 1].partie_imaginaire =
						(*((struct_complexe16 *)
						(*s_objet_1).objet)).partie_imaginaire;
			}
			else
			{
				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}
		}

		liberation(s_etat_processus, s_objet_1);

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_3) == d_erreur)
		{
			return;
		}

		if ((++(*((integer8 *) (*(*(*((struct_liste_chainee *)
				(*s_objet_2).objet)).suivant).donnee).objet))) > (integer8)
				(*((struct_matrice *) (*s_objet_3).objet)).nombre_colonnes)
		{
			(*((integer8 *) (*(*(*((struct_liste_chainee *) (*s_objet_2)
					.objet)).suivant).donnee).objet)) = 1;

			if ((++(*((integer8 *) (*(*((struct_liste_chainee *)
					(*s_objet_2).objet)).donnee).objet))) > (integer8)
					(*((struct_matrice *) (*s_objet_3).objet)).nombre_lignes)
			{
				(*((integer8 *) (*(*((struct_liste_chainee *)
						(*s_objet_2).objet)).donnee).objet)) = 1;
			}
		}

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_2) == d_erreur)
		{
			return;
		}
	}

/*
--------------------------------------------------------------------------------
  Traitement des lments des listes
--------------------------------------------------------------------------------
*/

	else if ((*s_objet_3).type == LST)
	{
		if ((*s_objet_2).type != INT)
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
			return;
		}

		indice_i = (*((integer8 *) (*s_objet_2).objet));
		indice_j = 1;

		if ((*s_objet_3).nombre_occurrences > 1)
		{
			if ((s_copie_3 = copie_objet(s_etat_processus, s_objet_3, 'N'))
					== NULL)
			{
				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			liberation(s_etat_processus, s_objet_3);
			s_objet_3 = s_copie_3;
		}

		l_element_courant = (*s_objet_3).objet;
		nombre_elements = 0;

		while(l_element_courant != NULL)
		{
			l_element_courant = (*l_element_courant).suivant;
			nombre_elements++;
		}

		l_element_courant = (*s_objet_3).objet;

		while((l_element_courant != NULL) && (indice_j != indice_i))
		{
			l_element_courant = (*l_element_courant).suivant;
			indice_j++;
		}

		if (l_element_courant != NULL)
		{
			liberation(s_etat_processus, (*l_element_courant).donnee);
			(*l_element_courant).donnee = s_objet_1;
		}
		else
		{
			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
			return;
		}

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_3) == d_erreur)
		{
			return;
		}

		(*((integer8 *) (*s_objet_2).objet)) =
				(indice_i % nombre_elements) + 1;

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_2) == d_erreur)
		{
			return;
		}
	}

/*
--------------------------------------------------------------------------------
  Traitement des noms
--------------------------------------------------------------------------------
*/

	else if ((*s_objet_3).type == NOM)
	{
		variable_partagee = d_faux;

		if (recherche_variable(s_etat_processus, (*((struct_nom *)
				(*s_objet_3).objet)).nom) == d_faux)
		{
			(*s_etat_processus).erreur_systeme = d_es;
			(*s_etat_processus).erreur_execution = d_ex_variable_non_definie;

			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			return;
		}

		if ((*(*s_etat_processus).pointeur_variable_courante)
				.variable_verrouillee == d_vrai)
		{
			(*s_etat_processus).erreur_execution = d_ex_variable_verrouillee;

			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			return;
		}

		s_objet_4 = (*(*s_etat_processus).pointeur_variable_courante).objet;

		if (s_objet_4 == NULL)
		{
			if (recherche_variable_partagee(s_etat_processus,
					(*(*s_etat_processus).pointeur_variable_courante).nom,
					(*(*s_etat_processus).pointeur_variable_courante)
					.variable_partagee, (*(*s_etat_processus)
					.pointeur_variable_courante).origine) == NULL)
			{
				if (pthread_mutex_unlock(&((*(*s_etat_processus)
						.pointeur_variable_partagee_courante).mutex)) != 0)
				{
					(*s_etat_processus).erreur_systeme = d_es_processus;
					return;
				}

				(*s_etat_processus).erreur_systeme = d_es;
				(*s_etat_processus).erreur_execution =
						d_ex_variable_non_definie;

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				return;
			}

			s_objet_4 = (*(*s_etat_processus)
					.pointeur_variable_partagee_courante).objet;
			variable_partagee = d_vrai;
		}

		if (((*s_objet_4).type == VIN) ||
				((*s_objet_4).type == VRL) ||
				((*s_objet_4).type == VCX))
		{
			if ((*s_objet_2).type != LST)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}

			l_element_courant = (*s_objet_2).objet;
			nombre_dimensions = 0;

			while(l_element_courant != NULL)
			{
				nombre_dimensions++;
				l_element_courant = (*l_element_courant).suivant;
			}

			if (nombre_dimensions != 1)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_dimensions_invalides;
				return;
			}

			l_element_courant = (*s_objet_2).objet;

			if ((*(*l_element_courant).donnee).type != INT)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}

			if ((*((integer8 *) (*(*l_element_courant).donnee).objet)) <= 0)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution = d_ex_argument_invalide;
				return;
			}
			else if ((*((integer8 *) (*(*l_element_courant).donnee).objet)) >
					(integer8) (*((struct_vecteur *) (*s_objet_4).objet))
					.taille)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
				return;
			}

			indice_i = (*((integer8 *) (*(*l_element_courant).donnee).objet));

			if ((s_copie_4 = copie_objet(s_etat_processus, s_objet_4, 'Q'))
					== NULL)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			liberation(s_etat_processus, s_objet_4);
			s_objet_4 = s_copie_4;

			if ((*s_objet_4).type == VIN)
			{
				/*
				 * Vecteur d'entiers
				 */

				if ((*s_objet_1).type == INT)
				{
					/*
					 * Aucune conversion de type
					 */

					((integer8 *) (*((struct_vecteur *) (*s_objet_4).objet))
							.tableau)[indice_i - 1] = (*((integer8 *)
							(*s_objet_1).objet));
				}
				else if ((*s_objet_1).type == REL)
				{
					/*
					 * Conversion du vecteur en vecteur rel
					 */

					tampon = (void *) ((integer8 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau);

					(*((struct_vecteur *) (*s_objet_4).objet)).type = 'R';
					(*s_objet_4).type = VRL;

					if (((*((struct_vecteur *) (*s_objet_4).objet)).tableau
							= malloc(((size_t) (*((struct_vecteur *)
							(*s_objet_4).objet)).taille) * sizeof(real8)))
							== NULL)
					{
						if (variable_partagee == d_vrai)
						{
							if (pthread_mutex_unlock(&((*(*s_etat_processus)
									.pointeur_variable_partagee_courante)
									.mutex)) != 0)
							{
								(*s_etat_processus).erreur_systeme =
										d_es_processus;
								return;
							}
						}

						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(i = 0; i < (*((struct_vecteur *) (*s_objet_4).objet))
							.taille; i++)
					{
						((real8 *) (*((struct_vecteur *) (*s_objet_4).objet))
								.tableau)[i] = (real8) (((integer8 *)
								tampon)[i]);
					}

					free((integer8 *) tampon);

					((real8 *) (*((struct_vecteur *) (*s_objet_4).objet))
							.tableau)[indice_i - 1] = (*((real8 *)
							(*s_objet_1).objet));
				}
				else if ((*s_objet_1).type == CPL)
				{
					/*
					 * Conversion du vecteur en vecteur complexe
					 */

					tampon = (void *) ((integer8 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau);

					(*((struct_vecteur *) (*s_objet_4).objet)).type = 'C';
					(*s_objet_4).type = VCX;

					if (((*((struct_vecteur *) (*s_objet_4).objet)).tableau
							= malloc(((size_t) (*((struct_vecteur *)
							(*s_objet_4).objet)).taille) *
							sizeof(struct_complexe16))) == NULL)
					{
						if (variable_partagee == d_vrai)
						{
							if (pthread_mutex_unlock(&((*(*s_etat_processus)
									.pointeur_variable_partagee_courante)
									.mutex)) != 0)
							{
								(*s_etat_processus).erreur_systeme =
										d_es_processus;
								return;
							}
						}

						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(i = 0; i < (*((struct_vecteur *) (*s_objet_4).objet))
							.taille; i++)
					{
						((struct_complexe16 *) (*((struct_vecteur *)
								(*s_objet_4).objet)).tableau)[i].partie_reelle =
								(real8) (((integer8 *) tampon)[i]);
						((struct_complexe16 *) (*((struct_vecteur *)
								(*s_objet_4).objet)).tableau)[i]
								.partie_imaginaire = (real8) 0;
					}

					free((integer8 *) tampon);

					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							.partie_reelle = (*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_reelle;
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							.partie_imaginaire = (*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_imaginaire;
				}
				else
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}
			}
			else if ((*s_objet_4).type == VRL)
			{
				/*
				 * Vecteur de rels
				 */

				if ((*s_objet_1).type == INT)
				{
					/*
					 * Conversion de l'lment  insrer en rel
					 */

					((real8 *) (*((struct_vecteur *) (*s_objet_4).objet))
							.tableau)[indice_i - 1] = (real8) (*((integer8 *)
							(*s_objet_1).objet));
				}
				else if ((*s_objet_1).type == REL)
				{
					/*
					 * Aucune conversion de type
					 */

					((real8 *) (*((struct_vecteur *) (*s_objet_4).objet))
							.tableau)[indice_i - 1] = (*((real8 *)
							(*s_objet_1).objet));
				}
				else if ((*s_objet_1).type == CPL)
				{
					/*
					 * Conversion du vecteur en vecteur complexe
					 */

					tampon = (void *) ((integer8 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau);

					(*((struct_vecteur *) (*s_objet_4).objet)).type = 'C';
					(*s_objet_4).type = VCX;

					if (((*((struct_vecteur *) (*s_objet_4).objet)).tableau
							= malloc(((size_t) (*((struct_vecteur *)
							(*s_objet_4).objet)).taille) *
							sizeof(struct_complexe16))) == NULL)
					{
						if (variable_partagee == d_vrai)
						{
							if (pthread_mutex_unlock(&((*(*s_etat_processus)
									.pointeur_variable_partagee_courante)
									.mutex)) != 0)
							{
								(*s_etat_processus).erreur_systeme =
										d_es_processus;
								return;
							}
						}

						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(i = 0; i < (*((struct_vecteur *) (*s_objet_4).objet))
							.taille; i++)
					{
						((struct_complexe16 *) (*((struct_vecteur *)
								(*s_objet_4).objet)).tableau)[i].partie_reelle =
								((real8 *) tampon)[i];
						((struct_complexe16 *) (*((struct_vecteur *)
								(*s_objet_4).objet)).tableau)[i]
								.partie_imaginaire = (real8) 0;
					}

					free((real8 *) tampon);

					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							.partie_reelle = (*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_reelle;
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							.partie_imaginaire = (*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_imaginaire;
				}
				else
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante)
								.mutex)) != 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}
			}
			else
			{
				/*
				 * Vecteur de complexes
				 */

				if ((*s_objet_1).type == INT)
				{
					/*
					 * Conversion de l'lment  insrer en complexe
					 */

					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							.partie_reelle = (real8) (*((integer8 *)
							(*s_objet_1).objet));
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1] 
							.partie_imaginaire = (real8) 0;
				}
				else if ((*s_objet_1).type == REL)
				{
					/*
					 * Conversion de l'lment  insrer en complexe
					 */

					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							.partie_reelle = (*((real8 *) (*s_objet_1).objet));
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1] 
							.partie_imaginaire = (real8) 0;
				}
				else if ((*s_objet_1).type == CPL)
				{
					/*
					 * Aucune conversion de type
					 */

					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							.partie_reelle = (*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_reelle;
					((struct_complexe16 *) (*((struct_vecteur *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							.partie_imaginaire = (*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_imaginaire;
				}
				else
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante)
								.mutex)) != 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}
			}

			(*((integer8 *) (*(*l_element_courant).donnee).objet)) =
					(indice_i % (*((struct_vecteur *) (*s_objet_4).objet))
					.taille) + 1;

			if (variable_partagee == d_faux)
			{
				(*(*s_etat_processus).pointeur_variable_courante).objet =
						s_objet_4;
			}
			else
			{
				(*(*s_etat_processus).pointeur_variable_courante).objet = NULL;
				(*(*s_etat_processus)
						.pointeur_variable_partagee_courante).objet = s_objet_4;

				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}
			}

			liberation(s_etat_processus, s_objet_1);
		}
		else if (((*s_objet_4).type == MIN) ||
				((*s_objet_4).type == MRL) ||
				((*s_objet_4).type == MCX))
		{
			if ((*s_objet_2).type != LST)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}

			l_element_courant = (*s_objet_2).objet;
			nombre_dimensions = 0;

			while(l_element_courant != NULL)
			{
				nombre_dimensions++;
				l_element_courant = (*l_element_courant).suivant;
			}

			if (nombre_dimensions != 2)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_dimensions_invalides;
				return;
			}

			l_element_courant = (*s_objet_2).objet;

			indice_i = 0;
			indice_j = 0;

			while(l_element_courant != NULL)
			{
				if ((*(*l_element_courant).donnee).type != INT)
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme = d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}

				if ((*((integer8 *) (*(*l_element_courant).donnee).objet))
						<= 0)
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme = d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_argument_invalide;
					return;
				}

				if (indice_i == 0)
				{
					indice_i = (*((integer8 *)
							(*(*l_element_courant).donnee).objet));
				}
				else
				{
					indice_j = (*((integer8 *)
							(*(*l_element_courant).donnee).objet));
				}

				l_element_courant = (*l_element_courant).suivant;
			}

			if ((indice_i > (*((struct_matrice *) (*s_objet_4).objet))
					.nombre_lignes) || (indice_j > (*((struct_matrice *)
					(*s_objet_4).objet)).nombre_colonnes))
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
				return;
			}

			if ((s_copie_4 = copie_objet(s_etat_processus, s_objet_4, 'Q'))
					== NULL)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme = d_es_processus;
						return;
					}
				}

				(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
				return;
			}

			liberation(s_etat_processus, s_objet_4);
			s_objet_4 = s_copie_4;

			if ((*s_objet_4).type == MIN)
			{
				/*
				 * Matrice d'entiers
				 */

				if ((*s_objet_1).type == INT)
				{
					/*
					 * Aucune conversion de type
					 */

					((integer8 **) (*((struct_matrice *) (*s_objet_4).objet))
							.tableau)[indice_i - 1][indice_j - 1] =
							(*((integer8 *) (*s_objet_1).objet));
				}
				else if ((*s_objet_1).type == REL)
				{
					/*
					 * Conversion de la matrice en matrice relle
					 */

					tampon = (void *) ((integer8 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau);

					(*((struct_matrice *) (*s_objet_4).objet)).type = 'R';
					(*s_objet_4).type = MRL;

					if (((*((struct_matrice *) (*s_objet_4).objet)).tableau
							= malloc(((size_t) (*((struct_matrice *)
							(*s_objet_4).objet)).nombre_lignes) *
							sizeof(real8 *))) == NULL)
					{
						if (variable_partagee == d_vrai)
						{
							if (pthread_mutex_unlock(&((*(*s_etat_processus)
									.pointeur_variable_partagee_courante)
									.mutex)) != 0)
							{
								(*s_etat_processus).erreur_systeme =
										d_es_processus;
								return;
							}
						}

						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(i = 0; i < (*((struct_matrice *) (*s_objet_4).objet))
							.nombre_lignes; i++)
					{
						if ((((real8 **) (*((struct_matrice *)
								(*s_objet_4).objet)).tableau)[i]
								= malloc(((size_t) (*((struct_matrice *)
								(*s_objet_4).objet)).nombre_colonnes) *
								sizeof(real8))) == NULL)
						{
							if (variable_partagee == d_vrai)
							{
								if (pthread_mutex_unlock(&((*(*s_etat_processus)
										.pointeur_variable_partagee_courante)
										.mutex)) != 0)
								{
									(*s_etat_processus).erreur_systeme =
											d_es_processus;
									return;
								}
							}

							(*s_etat_processus).erreur_systeme =
									d_es_allocation_memoire;
							return;
						}

						for(j = 0; j < (*((struct_matrice *)
								(*s_objet_4).objet)).nombre_colonnes; j++)
						{
							((real8 **) (*((struct_matrice *) (*s_objet_4)
									.objet)).tableau)[i][j] = (real8)
									(((integer8 **) tampon)[i][j]);
						}

						free(((integer8 **) tampon)[i]);
					}

					free((integer8 **) tampon);

					((real8 **) (*((struct_matrice *) (*s_objet_4).objet))
							.tableau)[indice_i - 1][indice_j - 1] =
							(*((real8 *) (*s_objet_1).objet));
				}
				else if ((*s_objet_1).type == CPL)
				{
					/*
					 * Conversion de la matrice en matrice complexe
					 */

					tampon = (void *) ((integer8 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau);

					(*((struct_matrice *) (*s_objet_4).objet)).type = 'C';
					(*s_objet_4).type = MCX;

					if (((*((struct_matrice *) (*s_objet_4).objet)).tableau
							= malloc(((size_t) (*((struct_matrice *)
							(*s_objet_4).objet)).nombre_lignes) *
							sizeof(struct_complexe16 *))) == NULL)
					{
						if (variable_partagee == d_vrai)
						{
							if (pthread_mutex_unlock(&((*(*s_etat_processus)
									.pointeur_variable_partagee_courante)
									.mutex)) != 0)
							{
								(*s_etat_processus).erreur_systeme =
										d_es_processus;
								return;
							}
						}

						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(i = 0; i < (*((struct_matrice *) (*s_objet_4).objet))
							.nombre_lignes; i++)
					{
						if ((((struct_complexe16 **) (*((struct_matrice *)
								(*s_objet_4).objet)).tableau)[i]
								= malloc(((size_t) (*((struct_matrice *)
								(*s_objet_4).objet)).nombre_colonnes) *
								sizeof(struct_complexe16))) == NULL)
						{
							if (variable_partagee == d_vrai)
							{
								if (pthread_mutex_unlock(&((*(*s_etat_processus)
										.pointeur_variable_partagee_courante)
										.mutex)) != 0)
								{
									(*s_etat_processus).erreur_systeme =
											d_es_processus;
									return;
								}
							}

							(*s_etat_processus).erreur_systeme =
									d_es_allocation_memoire;
							return;
						}

						for(j = 0; j < (*((struct_matrice *)
								(*s_objet_4).objet)).nombre_colonnes; j++)
						{
							((struct_complexe16 **) (*((struct_matrice *)
									(*s_objet_4).objet)).tableau)[i][j]
									.partie_reelle = (real8) (((integer8 **)
									tampon)[i][j]);
							((struct_complexe16 **) (*((struct_matrice *)
									(*s_objet_4).objet)).tableau)[i][j]
									.partie_imaginaire = (real8) 0;
						}

						free(((integer8 **) tampon)[i]);
					}

					free((integer8 **) tampon);

					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							[indice_j - 1].partie_reelle =
							(*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_reelle;
					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							[indice_j - 1].partie_imaginaire =
							(*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_imaginaire;
				}
				else
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}
			}
			else if ((*s_objet_4).type == MRL)
			{
				/*
				 * Matrice de rels
				 */

				if ((*s_objet_1).type == INT)
				{
					/*
					 * Conversion de l'lment  insrer en rel
					 */

					((real8 **) (*((struct_matrice *) (*s_objet_4).objet))
							.tableau)[indice_i - 1][indice_j - 1] =
							(real8) (*((integer8 *) (*s_objet_1).objet));
				}
				else if ((*s_objet_1).type == REL)
				{
					/*
					 * Aucune conversion de type
					 */

					((real8 **) (*((struct_matrice *) (*s_objet_4).objet))
							.tableau)[indice_i - 1][indice_j - 1] =
							(*((real8 *) (*s_objet_1).objet));
				}
				else if ((*s_objet_1).type == CPL)
				{
					/*
					 * Conversion de la matrice en matrice complexe
					 */

					tampon = (void *) ((real8 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau);

					(*((struct_matrice *) (*s_objet_4).objet)).type = 'C';
					(*s_objet_4).type = MCX;

					if (((*((struct_matrice *) (*s_objet_4).objet)).tableau
							= malloc(((size_t) (*((struct_matrice *)
							(*s_objet_4).objet)).nombre_lignes) *
							sizeof(struct_complexe16 *))) == NULL)
					{
						if (variable_partagee == d_vrai)
						{
							if (pthread_mutex_unlock(&((*(*s_etat_processus)
									.pointeur_variable_partagee_courante)
									.mutex)) != 0)
							{
								(*s_etat_processus).erreur_systeme =
										d_es_processus;
								return;
							}
						}

						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					for(i = 0; i < (*((struct_matrice *) (*s_objet_4).objet))
							.nombre_lignes; i++)
					{
						if ((((struct_complexe16 **) (*((struct_matrice *)
								(*s_objet_4).objet)).tableau)[i]
								= malloc(((size_t) (*((struct_matrice *)
								(*s_objet_4).objet)).nombre_colonnes) *
								sizeof(struct_complexe16))) == NULL)
						{
							if (variable_partagee == d_vrai)
							{
								if (pthread_mutex_unlock(&((*(*s_etat_processus)
										.pointeur_variable_partagee_courante)
										.mutex)) != 0)
								{
									(*s_etat_processus).erreur_systeme =
											d_es_processus;
									return;
								}
							}

							(*s_etat_processus).erreur_systeme =
									d_es_allocation_memoire;
							return;
						}

						for(j = 0; j < (*((struct_matrice *)
								(*s_objet_4).objet)).nombre_colonnes; j++)
						{
							((struct_complexe16 **) (*((struct_matrice *)
									(*s_objet_4).objet)).tableau)[i][j]
									.partie_reelle = (((real8 **)
									tampon)[i][j]);
							((struct_complexe16 **) (*((struct_matrice *)
									(*s_objet_4).objet)).tableau)[i][j]
									.partie_imaginaire = (real8) 0;
						}

						free(((integer8 **) tampon)[i]);
					}

					free((integer8 **) tampon);

					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							[indice_j - 1].partie_reelle =
							(*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_reelle;
					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							[indice_j - 1].partie_imaginaire =
							(*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_imaginaire;
				}
				else
				{ 
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}
			}
			else
			{
				/*
				 * Matrice de complexes
				 */

				if ((*s_objet_1).type == INT)
				{
					/*
					 * Conversion de l'lment  insrer en complexe
					 */

					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							[indice_j - 1].partie_reelle = (real8)
							(*((integer8 *) (*s_objet_1).objet));
					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1] 
							[indice_j - 1].partie_imaginaire = (real8) 0;
				}
				else if ((*s_objet_1).type == REL)
				{
					/*
					 * Conversion de l'lment  insrer en complexe
					 */

					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							[indice_j - 1].partie_reelle =
							(*((real8 *) (*s_objet_1).objet));
					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1] 
							[indice_j - 1].partie_imaginaire = (real8) 0;
				}
				else if ((*s_objet_1).type == CPL)
				{
					/*
					 * Aucune conversion de type
					 */

					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							[indice_j - 1].partie_reelle =
							(*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_reelle;
					((struct_complexe16 **) (*((struct_matrice *)
							(*s_objet_4).objet)).tableau)[indice_i - 1]
							[indice_j - 1].partie_imaginaire =
							(*((struct_complexe16 *)
							(*s_objet_1).objet)).partie_imaginaire;
				}
				else
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					liberation(s_etat_processus, s_objet_1);
					liberation(s_etat_processus, s_objet_2);
					liberation(s_etat_processus, s_objet_3);

					(*s_etat_processus).erreur_execution =
							d_ex_erreur_type_argument;
					return;
				}
			}

			if ((++(*((integer8 *) (*(*(*((struct_liste_chainee *)
					(*s_objet_2).objet)).suivant).donnee).objet))) > (integer8)
					(*((struct_matrice *) (*s_objet_4).objet)).nombre_colonnes)
			{
				(*((integer8 *) (*(*(*((struct_liste_chainee *) (*s_objet_2)
						.objet)).suivant).donnee).objet)) = 1;

				if ((++(*((integer8 *) (*(*((struct_liste_chainee *)
						(*s_objet_2).objet)).donnee).objet))) > (integer8)
						(*((struct_matrice *) (*s_objet_4).objet))
						.nombre_lignes)
				{
					(*((integer8 *) (*(*((struct_liste_chainee *)
							(*s_objet_2).objet)).donnee).objet)) = 1;
				}
			}

			if (variable_partagee == d_faux)
			{
				(*(*s_etat_processus).pointeur_variable_courante).objet =
						s_objet_4;
			}
			else
			{
				(*(*s_etat_processus).pointeur_variable_courante).objet = NULL;
				(*(*s_etat_processus).pointeur_variable_partagee_courante)
						.objet = s_objet_4;

				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_processus;
						return;
					}
				}
			}

			liberation(s_etat_processus, s_objet_1);
		}
		else if ((*s_objet_4).type == LST)
		{
			if ((*s_objet_2).type != INT)
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}

			indice_i = (*((integer8 *) (*s_objet_2).objet));
			indice_j = 1;

			if ((*s_objet_4).nombre_occurrences > 1)
			{
				if ((s_copie_4 = copie_objet(s_etat_processus, s_objet_4, 'N'))
						== NULL)
				{
					if (variable_partagee == d_vrai)
					{
						if (pthread_mutex_unlock(&((*(*s_etat_processus)
								.pointeur_variable_partagee_courante).mutex))
								!= 0)
						{
							(*s_etat_processus).erreur_systeme =
									d_es_processus;
							return;
						}
					}

					(*s_etat_processus).erreur_systeme =
							d_es_allocation_memoire;
					return;
				}

				liberation(s_etat_processus, s_objet_4);
				s_objet_4 = s_copie_4;
			}

			l_element_courant = (*s_objet_4).objet;
			nombre_elements = 0;

			while(l_element_courant != NULL)
			{
				l_element_courant = (*l_element_courant).suivant;
				nombre_elements++;
			}

			l_element_courant = (*s_objet_4).objet;

			while((l_element_courant != NULL) && (indice_j != indice_i))
			{
				l_element_courant = (*l_element_courant).suivant;
				indice_j++;
			}

			if (l_element_courant != NULL)
			{
				liberation(s_etat_processus, (*l_element_courant).donnee);
				(*l_element_courant).donnee = s_objet_1;
			}
			else
			{
				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_processus;
						return;
					}
				}

				liberation(s_etat_processus, s_objet_1);
				liberation(s_etat_processus, s_objet_2);
				liberation(s_etat_processus, s_objet_3);

				(*s_etat_processus).erreur_execution = d_ex_element_inexistant;
				return;
			}

			(*((integer8 *) (*s_objet_2).objet)) =
					(indice_i % nombre_elements) + 1;

			if (variable_partagee == d_faux)
			{
				(*(*s_etat_processus).pointeur_variable_courante).objet =
						s_objet_4;
			}
			else
			{
				(*(*s_etat_processus).pointeur_variable_courante).objet = NULL;
				(*(*s_etat_processus).pointeur_variable_partagee_courante)
						.objet = s_objet_4;

				if (variable_partagee == d_vrai)
				{
					if (pthread_mutex_unlock(&((*(*s_etat_processus)
							.pointeur_variable_partagee_courante).mutex)) != 0)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_processus;
						return;
					}
				}
			}
		}
		else
		{
			if (variable_partagee == d_vrai)
			{
				if (pthread_mutex_unlock(&((*(*s_etat_processus)
						.pointeur_variable_partagee_courante).mutex)) != 0)
				{
					(*s_etat_processus).erreur_systeme =
							d_es_processus;
					return;
				}
			}

			liberation(s_etat_processus, s_objet_1);
			liberation(s_etat_processus, s_objet_2);
			liberation(s_etat_processus, s_objet_3);

			(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
			return;
		}

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_3) == d_erreur)
		{
			return;
		}

		if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
				s_objet_2) == d_erreur)
		{
			return;
		}
	}

/*
--------------------------------------------------------------------------------
  Arguments incompatibles
--------------------------------------------------------------------------------
*/

	else
	{
		liberation(s_etat_processus, s_objet_1);
		liberation(s_etat_processus, s_objet_2);
		liberation(s_etat_processus, s_objet_3);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	return;
}

// vim: ts=4
