/*
================================================================================
  RPL/2 (R) version 4.1.21
  Copyright (C) 1989-2015 Dr. BERTRAND Jol

  This file is part of RPL/2.

  RPL/2 is free software; you can redistribute it and/or modify it
  under the terms of the CeCILL V2 License as published by the french
  CEA, CNRS and INRIA.
 
  RPL/2 is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the CeCILL V2 License
  for more details.
 
  You should have received a copy of the CeCILL License
  along with RPL/2. If not, write to info@cecill.info.
================================================================================
*/


#include "rpl-conv.h"


/*
================================================================================
  Fonction 'restart'
================================================================================
  Entres : pointeur sur une structure struct_processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_restart(struct_processus *s_etat_processus)
{
	struct timespec						attente;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  RESTART ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(rinitialisation du programme)\n\n");
			printf("  Aucun argument\n");
		}
		else
		{
			printf("(program reinitialization)\n\n");
			printf("  No argument\n");
		}

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}
	
	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 0) == d_erreur)
		{
			return;
		}
	}

	envoi_signal_processus((*s_etat_processus).pid_processus_pere,
			rpl_sigabort);
	(*s_etat_processus).requete_arret = 'Y';

	attente.tv_sec = 0;
	attente.tv_nsec = GRANULARITE_us * 1000;

	while((*s_etat_processus).var_volatile_requete_arret == 0)
	{
		scrutation_interruptions(s_etat_processus);
		nanosleep(&attente, NULL);
		INCR_GRANULARITE(attente.tv_nsec);
	}

	if ((*s_etat_processus).traitement_instruction_halt == d_vrai)
	{
		(*s_etat_processus).execution_pas_suivant = d_vrai;
	}

	(*s_etat_processus).requete_redemarrage = d_vrai;
	return;
}


/*
================================================================================
  Fonction 'regex'
================================================================================
  Entres : pointeur sur une structure struct_processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_regex(struct_processus *s_etat_processus)
{
	regex_t							expression;

	struct_liste_chainee			*l_element_courant_ecriture;
	struct_liste_chainee			*l_element_courant_lecture;

	struct_objet					*s_objet_argument_1;
	struct_objet					*s_objet_argument_2;
	struct_objet					*s_objet_resultat;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  REGEX ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(expression rgulire)\n\n");
		}
		else
		{
			printf("(regular expression)\n\n");
		}

		printf("    2: %s\n", d_LST);
		printf("    1: %s\n", d_CHN);
		printf("->  1: %s\n\n", d_LST);

		printf("    2: %s\n", d_CHN);
		printf("    1: %s\n", d_CHN);
		printf("->  1: %s (0 or -1)\n", d_INT);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 2) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_1) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_2) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_argument_1);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (((*s_objet_argument_1).type == CHN) &&
			((*s_objet_argument_2).type == CHN))
	{
		// Renvoie 0 ou -1

		if (regcomp(&expression, (const char *) (*s_objet_argument_1).objet,
				REG_EXTENDED | REG_NOSUB) != 0)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution = d_ex_expression_reguliere;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, INT)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (regexec(&expression, (const char *) (*s_objet_argument_2).objet,
				0, NULL, 0) == 0)
		{
			(*((integer8 *) (*s_objet_resultat).objet)) = -1;
		}
		else // REG_NOMATCH
		{
			(*((integer8 *) (*s_objet_resultat).objet)) = 0;
		}

		regfree(&expression);
	}
	else if (((*s_objet_argument_1).type == CHN) &&
			((*s_objet_argument_2).type == LST))
	{
		// Renvoie une liste de chanes qui correspondent

		if (regcomp(&expression, (const char *) (*s_objet_argument_1).objet,
				REG_EXTENDED | REG_NOSUB) != 0)
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);

			(*s_etat_processus).erreur_execution = d_ex_expression_reguliere;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, LST)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		l_element_courant_lecture = (*s_objet_argument_2).objet;
		l_element_courant_ecriture = NULL;

		while(l_element_courant_lecture != NULL)
		{
			if ((*(*l_element_courant_lecture).donnee).type != CHN)
			{
				regfree(&expression);
				liberation(s_etat_processus, s_objet_argument_1);
				liberation(s_etat_processus, s_objet_argument_2);
				liberation(s_etat_processus, s_objet_resultat);

				(*s_etat_processus).erreur_execution =
						d_ex_erreur_type_argument;
				return;
			}

			if (regexec(&expression, (const char *)
					(*(*l_element_courant_lecture).donnee).objet,
					0, NULL, 0) == 0)
			{
				if (l_element_courant_ecriture == NULL)
				{
					if (((*s_objet_resultat).objet =
							allocation_maillon(s_etat_processus)) == NULL)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					l_element_courant_ecriture = (*s_objet_resultat).objet;

					if (((*l_element_courant_ecriture).donnee =
							copie_objet(s_etat_processus,
							(*l_element_courant_lecture).donnee, 'P'))
							== NULL)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					(*l_element_courant_ecriture).suivant = NULL;
				}
				else
				{
					if (((*l_element_courant_ecriture).suivant =
							allocation_maillon(s_etat_processus)) == NULL)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					l_element_courant_ecriture =
							(*l_element_courant_ecriture).suivant;

					if (((*l_element_courant_ecriture).donnee =
							copie_objet(s_etat_processus,
							(*l_element_courant_lecture).donnee, 'P'))
							== NULL)
					{
						(*s_etat_processus).erreur_systeme =
								d_es_allocation_memoire;
						return;
					}

					(*l_element_courant_ecriture).suivant = NULL;
				}
			}

			l_element_courant_lecture = (*l_element_courant_lecture).suivant;
		}

		regfree(&expression);
	}
	else
	{
		liberation(s_etat_processus, s_objet_argument_1);
		liberation(s_etat_processus, s_objet_argument_2);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument_1);
	liberation(s_etat_processus, s_objet_argument_2);

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_resultat) == d_erreur)
	{
		return;
	}

	return;
}


/*
================================================================================
  Fonction 'rgdl'
================================================================================
  Entres : pointeur sur une structure struct_processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_rgdl(struct_processus *s_etat_processus)
{
	integer8						i;
	integer8						l;

	struct_objet					*s_objet_argument_1;
	struct_objet					*s_objet_argument_2;
	struct_objet					*s_objet_argument_3;
	struct_objet					*s_objet_resultat;

	unsigned char					*ptr_e;
	unsigned char					*ptr_l;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  RGDL ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(fer  droite)\n\n");
		}
		else
		{
			printf("(ragged left)\n\n");
		}

		printf("    3: %s\n", d_CHN);
		printf("    2: %s\n", d_INT);
		printf("    1: %s\n", d_CHN);
		printf("->  1: %s\n", d_CHN);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 3) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_1) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_2) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_argument_1);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_3) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_argument_1);
		liberation(s_etat_processus, s_objet_argument_2);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (((*s_objet_argument_1).type == CHN) &&
			((*s_objet_argument_2).type == INT) &&
			((*s_objet_argument_3).type == CHN))
	{
		if ((strlen((unsigned char *) (*s_objet_argument_1).objet) != 1) ||
				((*((integer8 *) (*s_objet_argument_2).objet)) < 0))
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);
			liberation(s_etat_processus, s_objet_argument_3);

			(*s_etat_processus).erreur_execution = d_ex_argument_invalide;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, CHN)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*s_objet_resultat).objet = malloc(((size_t) ((*((integer8 *)
				(*s_objet_argument_2).objet)) + 1)) * sizeof(unsigned char)))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((l = (integer8) strlen((unsigned char *) (*s_objet_argument_3)
				.objet))) >= (*((integer8 *) (*s_objet_argument_2).objet)))
		{
			// Recadrage simple

			ptr_e = (unsigned char *) (*s_objet_resultat).objet;
			ptr_l = ((unsigned char *) (*s_objet_argument_3).objet)
					+ (l - (*((integer8 *) (*s_objet_argument_2).objet)));

			while((*ptr_l) != d_code_fin_chaine)
			{
				*ptr_e++ = *ptr_l++;
			}

			(*ptr_e) = d_code_fin_chaine;
		}
		else
		{
			// Ajout des caractres

			ptr_e = (unsigned char *) (*s_objet_resultat).objet;
			ptr_l = (unsigned char *) (*s_objet_argument_3).objet;

			for(i = 0; i < (*((integer8 *) (*s_objet_argument_2).objet)) - l;
					i++)
			{
				*ptr_e++ = ((unsigned char *) (*s_objet_argument_1).objet)[0];
			}

			while((*ptr_l) != d_code_fin_chaine)
			{
				*ptr_e++ = *ptr_l++;
			}

			(*ptr_e) = d_code_fin_chaine;
		}
	}
	else
	{
		liberation(s_etat_processus, s_objet_argument_1);
		liberation(s_etat_processus, s_objet_argument_2);
		liberation(s_etat_processus, s_objet_argument_3);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument_1);
	liberation(s_etat_processus, s_objet_argument_2);
	liberation(s_etat_processus, s_objet_argument_3);

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_resultat) == d_erreur)
	{
		return;
	}

	return;
}


/*
================================================================================
  Fonction 'rgdr'
================================================================================
  Entres : pointeur sur une structure struct_processus
--------------------------------------------------------------------------------
  Sorties :
--------------------------------------------------------------------------------
  Effets de bord : nant
================================================================================
*/

void
instruction_rgdr(struct_processus *s_etat_processus)
{
	integer8						i;
	integer8						l;

	struct_objet					*s_objet_argument_1;
	struct_objet					*s_objet_argument_2;
	struct_objet					*s_objet_argument_3;
	struct_objet					*s_objet_resultat;

	unsigned char					*ptr_e;
	unsigned char					*ptr_l;

	(*s_etat_processus).erreur_execution = d_ex;

	if ((*s_etat_processus).affichage_arguments == 'Y')
	{
		printf("\n  RGDR ");

		if ((*s_etat_processus).langue == 'F')
		{
			printf("(fer  gauche)\n\n");
		}
		else
		{
			printf("(ragged right)\n\n");
		}

		printf("    3: %s\n", d_CHN);
		printf("    2: %s\n", d_INT);
		printf("    1: %s\n", d_CHN);
		printf("->  1: %s\n", d_CHN);

		return;
	}
	else if ((*s_etat_processus).test_instruction == 'Y')
	{
		(*s_etat_processus).nombre_arguments = -1;
		return;
	}

	if (test_cfsf(s_etat_processus, 31) == d_vrai)
	{
		if (empilement_pile_last(s_etat_processus, 3) == d_erreur)
		{
			return;
		}
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_1) == d_erreur)
	{
		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_2) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_argument_1);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (depilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			&s_objet_argument_3) == d_erreur)
	{
		liberation(s_etat_processus, s_objet_argument_1);
		liberation(s_etat_processus, s_objet_argument_2);

		(*s_etat_processus).erreur_execution = d_ex_manque_argument;
		return;
	}

	if (((*s_objet_argument_1).type == CHN) &&
			((*s_objet_argument_2).type == INT) &&
			((*s_objet_argument_3).type == CHN))
	{
		if ((strlen((unsigned char *) (*s_objet_argument_1).objet) != 1) ||
				((*((integer8 *) (*s_objet_argument_2).objet)) < 0))
		{
			liberation(s_etat_processus, s_objet_argument_1);
			liberation(s_etat_processus, s_objet_argument_2);
			liberation(s_etat_processus, s_objet_argument_3);

			(*s_etat_processus).erreur_execution = d_ex_argument_invalide;
			return;
		}

		if ((s_objet_resultat = allocation(s_etat_processus, CHN)) == NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((*s_objet_resultat).objet = malloc(((size_t) ((*((integer8 *)
				(*s_objet_argument_2).objet)) + 1)) * sizeof(unsigned char)))
				== NULL)
		{
			(*s_etat_processus).erreur_systeme = d_es_allocation_memoire;
			return;
		}

		if (((l = (integer8) strlen((unsigned char *) (*s_objet_argument_3)
				.objet))) >= (*((integer8 *) (*s_objet_argument_2).objet)))
		{
			// Recadrage simple

			ptr_e = (unsigned char *) (*s_objet_resultat).objet;
			ptr_l = ((unsigned char *) (*s_objet_argument_3).objet);

			for(i = 0; i < (*((integer8 *) (*s_objet_argument_2).objet)); i++)
			{
				*ptr_e++ = *ptr_l++;
			}

			(*ptr_e) = d_code_fin_chaine;
		}
		else
		{
			// Ajout des caractres

			ptr_e = (unsigned char *) (*s_objet_resultat).objet;
			ptr_l = (unsigned char *) (*s_objet_argument_3).objet;

			while((*ptr_l) != d_code_fin_chaine)
			{
				*ptr_e++ = *ptr_l++;
			}

			for(i = 0; i < (*((integer8 *) (*s_objet_argument_2).objet)) - l;
					i++)
			{
				*ptr_e++ = ((unsigned char *) (*s_objet_argument_1).objet)[0];
			}

			(*ptr_e) = d_code_fin_chaine;
		}
	}
	else
	{
		liberation(s_etat_processus, s_objet_argument_1);
		liberation(s_etat_processus, s_objet_argument_2);
		liberation(s_etat_processus, s_objet_argument_3);

		(*s_etat_processus).erreur_execution = d_ex_erreur_type_argument;
		return;
	}

	liberation(s_etat_processus, s_objet_argument_1);
	liberation(s_etat_processus, s_objet_argument_2);
	liberation(s_etat_processus, s_objet_argument_3);

	if (empilement(s_etat_processus, &((*s_etat_processus).l_base_pile),
			s_objet_resultat) == d_erreur)
	{
		return;
	}

	return;
}

// vim: ts=4
